/******************************************************************************
 * File: ObjectVector.impl.h
 * Date: 14.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ObjectVector.h"
#include "util/SerializationUtils.h"

#include <algorithm>

template<typename ObjType>
template<typename T>
void ObjectVector<ObjType>::deepCopy(const std::enable_if_t<std::is_copy_constructible<ObjType>::value, T>& src)
{
	this->clear();

	for (const auto& elem : src)
	{
		if (elem.get())
		{
			ObjType* obj = new ObjType{*elem.get()};
			this->emplace_back(obj);
		}
		else
			this->emplace_back(nullptr);
	}
}

template<typename ObjType>
void ObjectVector<ObjType>::erase(const object_type* obj)
{
	auto it = find(obj);

	if (it != this->cend())
		this->erase(it);
}

template<typename ObjType>
typename ObjectVector<ObjType>::vector_type ObjectVector<ObjType>::sorted() const
{
	auto sortedCopy = *this;
	std::sort(sortedCopy.begin(), sortedCopy.end(), [](const auto& obj1, const auto& obj2) { return *obj1 < *obj2; });
	return sortedCopy;
}

template<typename ObjType>
typename ObjectVector<ObjType>::pointer_type ObjectVector<ObjType>::selectFirst(std::function<bool(const pointer_type&)> pred) const
{
	auto it = std::find_if(this->cbegin(), this->cend(), pred);

	if (it != this->cend())
		return *it;

	return pointer_type{};
}

template<typename ObjType>
typename ObjectVector<ObjType>::pointer_vec_type ObjectVector<ObjType>::select(std::function<bool(const pointer_type&)> pred) const
{
	pointer_vec_type objects;
	std::copy_if(this->cbegin(), this->cend(), std::back_inserter(objects), pred);
	return objects;
}

template<typename ObjType>
auto ObjectVector<ObjType>::find(const pointer_type& obj) const
{
	return std::find(this->cbegin(), this->cend(), obj);
}

template<typename ObjType>
auto ObjectVector<ObjType>::find(const object_type* obj) const
{
	return std::find_if(this->cbegin(), this->cend(), [&obj](auto object) { return object.get() == obj; });
}

template<typename ObjType>
auto ObjectVector<ObjType>::find(std::function<bool(const pointer_type&)> pred) const
{
	return std::find_if(this->cbegin(), this->cend(), pred);
}

template<typename ObjType>
auto ObjectVector<ObjType>::find(std::function<bool(const object_type*)> pred) const
{
	return std::find_if(this->cbegin(), this->cend(), [&pred](auto object) { return pred(object.get()); });
}

template<typename ObjType>
auto ObjectVector<ObjType>::indexOf(const object_type* obj) const
{
	auto it = find(obj);

	if (it == this->cend())
		return -1;
	else
		return static_cast<int>(it - this->cbegin());
}

template<typename ObjType>
auto ObjectVector<ObjType>::indexOf(const pointer_type& obj) const
{
	return find(obj.get());
}

template<typename ObjType>
void ObjectVector<ObjType>::serialize(QString elemName, SerializationContext& ctx, std::function<bool(const pointer_type& object)> predicate) const
{
	SerializationUtils::serializeContainer(*this, elemName, ctx, predicate);
}

template<typename ObjType>
void ObjectVector<ObjType>::deserialize(QString elemName, DeserializationContext& ctx, std::function<pointer_type(const SettingsContainer&)> objCreator)
{
	SerializationUtils::deserializeContainer<vector_type>(elemName, ctx, objCreator);
}
