/******************************************************************************
 * File: ObjectProperty.h
 * Date: 17.4.2018
 *****************************************************************************/

#ifndef OBJECTPROPERTY_H
#define OBJECTPROPERTY_H

#include <memory>

#include "PropertyBase.h"

namespace grndr
{
	template<typename ObjType>
	class ObjectProperty : public PropertyBase
	{
	public:
		using property_type = ObjectProperty<ObjType>;
		using object_type = ObjType;

	public:
		ObjectProperty(PropertyID id, QString name, Flags flags = Flag::None);
		ObjectProperty(const property_type& src) = default;
		ObjectProperty(property_type&& src) = default;

		ObjectProperty& operator =(const property_type& src) = default;
		ObjectProperty& operator =(property_type&& src) = default;

	public:
		object_type& object() { return _object; }
		const object_type& object() const { return _object; }

		void objectModified() const { emit valueChanged(); }

	public:
		virtual void copyValue(const PropertyBase* property) override;

		virtual QString toString() const override { return ""; }
		virtual void fromString(const QString& data) override { Q_UNUSED(data); }

		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	protected:
		object_type _object{};
	};
}

#include "ObjectProperty.impl.h"

#endif
