/******************************************************************************
 * File: PropertyObject.h
 * Date: 20.3.2018
 *****************************************************************************/

#ifndef PROPERTYOBJECT_H
#define PROPERTYOBJECT_H

#include <QObject>

#include "common/properties/PropertyVector.h"
#include "common/properties/StandardProperties.h"

namespace grndr
{
	class PropertyObject : public QObject
	{
		Q_OBJECT

	public:
		PropertyVector& properties() { return _properties; }
		const PropertyVector& properties() const { return _properties; }

	public:
		virtual void serialize(SerializationContext& ctx) const;
		virtual void deserialize(DeserializationContext& ctx);

	protected:
		virtual void createProperties() { }

		template<typename PropType>
		std::shared_ptr<PropertyBase> createProperty(PropertyID id, QString name, PropertyBase::Flags flags = PropertyBase::Flag::None);
		template<typename PropType>
		std::shared_ptr<PropertyBase> createProperty(PropertyID id, QString name, typename PropType::value_type defValue, PropertyBase::Flags flags = PropertyBase::Flag::None);
		void removeProperty(PropertyID id);
		void removeProperty(const PropertyBase* prop);

	private:
		std::shared_ptr<PropertyBase> _createProperty(PropertyID id, std::function<std::shared_ptr<PropertyBase>()> creator);

	protected:
		PropertyVector _properties;
	};
}

#include "PropertyObject.impl.h"

#endif
