/******************************************************************************
 * File: SerializationContext.h
 * Date: 28.2.2018
 *****************************************************************************/

#ifndef SERIALIZATIONCONTEXT_H
#define SERIALIZATIONCONTEXT_H

#include <stack>

#include "SerializationBase.h"
#include "SettingsContainer.h"

namespace grndr
{
	class Project;
	class Block;
	class Port;	
	class Connection;
	class ImageReference;

	class SerializationContext : public SerializationBase
	{
	public:
		SerializationContext(QString rootName, Mode mode, const SettingsContainer* settings = nullptr);

	public:
		SettingsContainer& settings(bool rootSettings = false) { if (!rootSettings && !_settingsStack.empty()) return _settingsStack.top(); else return _settings; }
		const SettingsContainer& settings(bool rootSettings = false) const { if (!rootSettings && !_settingsStack.empty()) return _settingsStack.top(); else return _settings; }

	public:
		void beginGroup(QString name, bool isArray = false) { _settingsStack.emplace(name, isArray); }
		void endGroup();

	public:
		int addBlock(const Block* block) { return addObject(block, _blocks); }
		int getBlockIndex(const Block* block) const { return getObjectIndex(block, _blocks); }

		int addPort(const Port* port) { return addObject(port, _ports); }
		int getPortIndex(const Port* port) const { return getObjectIndex(port, _ports); }

		void addConnection(const Connection* con);
		const std::vector<const Connection*>& connections() { return _connections; }
		void clearConnections() { _connections.clear(); }

		int addImageReference(const ImageReference* imageRef) { return addObject(imageRef, _imageReferences); }
		int getImageReferenceIndex(const ImageReference* imageRef) const { return getObjectIndex(imageRef, _imageReferences); }

	private:
		template<typename ObjType>
		int addObject(const ObjType* obj, std::vector<const ObjType*>& vec);
		template<typename ObjType>
		int getObjectIndex(const ObjType* obj, const std::vector<const ObjType*>& vec) const;

	private:
		SettingsContainer _settings;
		std::stack<SettingsContainer> _settingsStack;

		std::vector<const Block*> _blocks;
		std::vector<const Port*> _ports;		
		std::vector<const Connection*> _connections;
		std::vector<const ImageReference*> _imageReferences;
	};
}

#include "SerializationContext.impl.h"

#endif
