/******************************************************************************
 * File: SettingsCodec.cpp
 * Date: 27.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "SettingsCodec.h"
#include "SerializationExceptions.h"

void SettingsCodec::saveContainer(const SettingsContainer& settings, QString fileName)
{
	try {
		QFile file{fileName};

		if (file.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text))
		{
			QTextStream out{&file};
			encodeSettings(settings, out);
		}
		else
			throw SerializationException{_EXCPT(QString{"Unable to open file '%1' for writing"}.arg(fileName))};
	} catch (SerializationException&) {
		// Just re-throw
		throw;
	} catch (std::exception& e) {
		// Forward any exceptions as a SerializationException
		throw SerializationException{_EXCPT(e.what())};
	}
}

void SettingsCodec::loadContainer(SettingsContainer& settings, QString fileName)
{
	try {
		QFile file{fileName};

		if (file.open(QIODevice::ReadOnly|QIODevice::Text))
		{
			QTextStream in{&file};
			decodeSettings(settings, in);
		}
		else
			throw SerializationException{_EXCPT(QString{"Unable to open file '%1' for reading"}.arg(fileName))};
	} catch (SerializationException&) {
		// Just re-throw
		throw;
	} catch (std::exception& e) {
		// Forward any exceptions as a SerializationException
		throw SerializationException{_EXCPT(e.what())};
	}
}
