/******************************************************************************
 * File: ProjectController.h
 * Date: 07.2.2018
 *****************************************************************************/

#ifndef PROJECTCONTROLLER_H
#define PROJECTCONTROLLER_H

#include <memory>

#include "GenericController.h"
#include "common/serialization/SettingsContainer.h"

namespace grndr
{
	class Project;
	class PipelineController;
	class Label;
	class LabelsListWidget;
	class ImageReference;
	class ImageReferencesListWidget;

	class ProjectController : public GenericController
	{
		Q_OBJECT

	public:
		ProjectController(Project* project);

	public:
		void assignUiComponents(LabelsListWidget* labelsList, ImageReferencesListWidget* imageRefsList);

	public:
		void clearProject(bool createDefaultLabel = false);
		void loadProject(QString fileName);
		void saveProject(QString fileName);

		QString getCurrentProjectFile() const { return _currentProjectFile; }
		bool isProjectDirty() const;

	public:
		void switchLabel(Label* label);
		Label* activeLabel() { return _activeLabel; }
		const Label* activeLabel() const { return _activeLabel; }

		void switchImageReference(ImageReference* imageRef);
		void switchToNextImageReference();
		void switchToPreviousImageReference();
		ImageReference* activeImageReference() { return _activeImageReference; }
		const ImageReference* activeImageReference() const { return _activeImageReference; }

	public:
		std::shared_ptr<Label> createLabel(QString name = "") const;
		void removeLabel(const Label* label);
		void removeAllLabels();
		void renameLabel(Label* label, QString newName) const;

		std::shared_ptr<ImageReference> createImageReference(QString imagePath) const;
		std::vector<std::shared_ptr<ImageReference>> createImageReferences(QStringList imagePaths, bool ignoreUnknownFiletypes = true) const;
		void removeImageReference(const ImageReference* imageRef);
		void removeAllImageReferences();

	signals:
		void projectClearing();
		void projectCleared();
		void projectLoading(QString);
		void projectLoaded(QString);
		void projectSaving(QString);
		void projectSaved(QString);

		void currentProjectFileChanged(QString fileName);

		void labelSwitching(Label*);
		void labelSwitched(Label*);
		void imageReferenceSwitching(ImageReference*);
		void imageReferenceSwitched(ImageReference*);

	private:
		void validateLabelName(QString name, const Label* label = nullptr) const;

		void setCurrentProjectFile(QString fileName);
		void updateCurrentProjectData();

	private slots:
		void labelCreated(const std::shared_ptr<Label>& label) const;
		void labelRemoved(const std::shared_ptr<Label>& label) const;

		void imageReferenceCreated(const std::shared_ptr<ImageReference>& imageRef) const;
		void imageReferenceRemoved(const std::shared_ptr<ImageReference>& imageRef) const;

	private:
		Project* _project{nullptr};
		QString _currentProjectFile{""};
		SettingsContainer _currentProjectSettings;

		LabelsListWidget* _labelsList{nullptr};
		ImageReferencesListWidget* _imageReferencesList{nullptr};

		Label* _activeLabel{nullptr};
		ImageReference* _activeImageReference{nullptr};
	};
}

#endif
