/******************************************************************************
 * File: ClipboardManager.impl.h
 * Date: 04.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ClipboardManager.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

template<typename ObjType>
void ClipboardManager::serialize(QString type, const std::vector<const ObjType*>& objects)
{
	std::vector<SettingsContainer> settings;

	// Serialize each object into an individual settings container
	for (auto object : objects)
	{
		SerializationContext ctx{type, SerializationContext::Mode::ClipboardSerialization};
		object->serialize(ctx);
		settings.push_back(ctx.settings());
	}

	setData(type, std::move(settings));
}

template<typename ObjType>
std::vector<ObjType*> ClipboardManager::deserialize(QString type, std::function<ObjType*(const SettingsContainer&)> objCreator) const
{
	std::vector<ObjType*> createdObjects;

	if (hasData(type))
	{
		for (const auto& objectData : getData(type))
		{
			if (auto object = objCreator(objectData))
			{
				DeserializationContext ctx{objectData, DeserializationContext::Mode::ClipboardSerialization};
				object->deserialize(ctx);
				createdObjects.push_back(object);
			}
		}
	}

	return createdObjects;
}
