/******************************************************************************
 * File: GrinderExceptions.cpp
 * Date: 17.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "GrinderExceptions.h"

#include <signal.h>

QString grndr::GetExceptionMessage(QString what, QString* func, QString* file, QString* line)
{
	QStringList tokens = what.split("|");

	if (func && tokens.size() >= 1)
		*func = tokens[0];

	if (file && tokens.size() >= 2)
		*file = tokens[1];

	if (line && tokens.size() >= 3)
		*line = tokens[2];

	return tokens.last();
}

void grndr::ShowExceptionMessage(QString what)
{
	QString str;

	if (!what.isEmpty())
	{
		QString func, file, line;
		what = GetExceptionMessage(what, &func, &file, &line);
		str = QString{"%1<br><em>%2 (%3:%4)</em>"}.arg(what).arg(func).arg(file).arg(line);
	}
	else
		str = "Unknown exception";

	QMessageBox::critical(nullptr, "Uncaught exception", QString{"<b>An unhandled exception occurred:</b><br><br>%1<br><br>The application will now close."}.arg(str));
}

void _GrinderSignalHandler(int signum)
{
	// Show an exception message and exit
	ShowExceptionMessage(_EXCPT(QString{"Received interrupt signal %1"}.arg(signum)));
	exit(signum);
}

void grndr::InstallSignalHandlers()
{
	// Catch all common interrupts
	signal(SIGINT, _GrinderSignalHandler);
	signal(SIGILL, _GrinderSignalHandler);
	signal(SIGABRT_COMPAT, _GrinderSignalHandler);
	signal(SIGFPE, _GrinderSignalHandler);
	signal(SIGSEGV, _GrinderSignalHandler);
	signal(SIGABRT, _GrinderSignalHandler);
	signal(SIGTERM, _GrinderSignalHandler);
}
