/******************************************************************************
 * File: GrinderSettings.cpp
 * Date: 16.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "GrinderSettings.h"

GrinderSettings::GrinderSettings(QString organization, QString application) : QSettings(organization, application)
{
	loadSettings();
}

void GrinderSettings::loadSettings()
{
	loadGeneralOptions();
	loadStartupOptions();
	loadImageOptions();
	loadImageEditorOptions();
	loadProjectFilesMRUList();
}

void GrinderSettings::saveSettings(bool notify)
{
	saveGeneralOptions();
	saveStartupOptions();
	saveImageOptions();
	saveImageEditorOptions();
	saveProjectFilesMRUList();

	if (notify)
		emit settingsChanged();
}

void GrinderSettings::getWindowState(QString name, QMainWindow* window)
{
	beginGroup("WindowStates");
	beginGroup(name);
	window->restoreGeometry(value("Geometry").toByteArray());
	window->restoreState(value("State").toByteArray());
	endGroup();
	endGroup();
}

void GrinderSettings::setWindowState(QString name, const QMainWindow* window)
{
	beginGroup("WindowStates");
	beginGroup(name);
	setValue("Geometry", window->saveGeometry());
	setValue("State", window->saveState());
	endGroup();
	endGroup();
}

void GrinderSettings::getSplitterState(QString name, QSplitter* splitter)
{
	beginGroup("SplitterStates");
	beginGroup(name);
	splitter->restoreState(value("State").toByteArray());
	endGroup();
	endGroup();
}

void GrinderSettings::setSplitterState(QString name, const QSplitter* splitter)
{
	beginGroup("SplitterStates");
	beginGroup(name);
	setValue("State", splitter->saveState());
	endGroup();
	endGroup();
}

QString GrinderSettings::getFileDialogDir(QString dlgName)
{
	beginGroup("FileDialogDirs");
	QString dir = value(dlgName, "").toString();
	endGroup();

	return dir;
}

void GrinderSettings::setFileDialogDir(QString dlgName, QString dir)
{
	beginGroup("FileDialogDirs");
	setValue(dlgName, dir);
	endGroup();
}

void GrinderSettings::loadGeneralOptions()
{
	beginGroup("GeneralOptions");
	_generalOptions.askOnUnsavedChanges = value("AskOnUnsavedChanges", true).toBool();
	endGroup();
}

void GrinderSettings::saveGeneralOptions()
{
	beginGroup("GeneralOptions");
	setValue("AskOnUnsavedChanges", _generalOptions.askOnUnsavedChanges);
	endGroup();
}

void GrinderSettings::loadStartupOptions()
{
	beginGroup("StartupOptions");
	_startupOptions.loadLastProject = value("LoadLastProject", false).toBool();
	_startupOptions.lastProjectFile = value("LastProjectFile", "").toString();
	endGroup();
}

void GrinderSettings::saveStartupOptions()
{
	beginGroup("StartupOptions");
	setValue("LoadLastProject", _startupOptions.loadLastProject);
	setValue("LastProjectFile", _startupOptions.lastProjectFile);
	endGroup();
}

void GrinderSettings::loadImageOptions()
{
	beginGroup("ImageOptions");
	_imageOptions.additionalSearchPaths = value("AdditionalSearchPaths", "").toStringList();
	endGroup();
}

void GrinderSettings::saveImageOptions()
{
	beginGroup("ImageOptions");
	setValue("AdditionalSearchPaths", _imageOptions.additionalSearchPaths);
	endGroup();
}

void GrinderSettings::loadImageEditorOptions()
{
	beginGroup("StartupOptions");
	_imageEditorOptions.groupIntoTabs = value("GroupIntoTabs", true).toBool();
	_imageEditorOptions.startUndocked = value("StartUndocked", false).toBool();
	_imageEditorOptions.autoFitToWindow = value("AutoFitToWindow", false).toBool();
	endGroup();
}

void GrinderSettings::saveImageEditorOptions()
{
	beginGroup("StartupOptions");
	setValue("GroupIntoTabs", _imageEditorOptions.groupIntoTabs);
	setValue("StartUndocked", _imageEditorOptions.startUndocked );
	setValue("AutoFitToWindow", _imageEditorOptions.autoFitToWindow);
	endGroup();
}

void GrinderSettings::loadProjectFilesMRUList()
{
	beginGroup("MRULists");
	_recentProjects = value("ProjectFiles").toStringList();
	endGroup();
}

void GrinderSettings::saveProjectFilesMRUList()
{
	beginGroup("MRULists");
	setValue("ProjectFiles", static_cast<QStringList>(_recentProjects));
	endGroup();
}
