/******************************************************************************
 * File: GrinderSettings.h
 * Date: 16.2.2018
 *****************************************************************************/

#ifndef GRINDERSETTINGS_H
#define GRINDERSETTINGS_H

#include <QSettings>

#include "common/MRUStringList.h"

namespace grndr
{
	class GrinderSettings : public QSettings
	{
		Q_OBJECT

	public:
		struct GeneralOptions
		{
			bool askOnUnsavedChanges{true};
		};

		struct StartupOptions
		{
			bool loadLastProject{false};
			QString lastProjectFile{""};
		};

		struct ImageOptions
		{
			QStringList additionalSearchPaths;
		};

		struct ImageEditorOptions
		{
			bool groupIntoTabs{true};
			bool startUndocked{false};

			bool autoFitToWindow{false};
		};

	public:
		GrinderSettings(QString organization, QString application);
		virtual ~GrinderSettings() { saveSettings(false); }

	public:
		void loadSettings();
		void saveSettings(bool notify = true);

	public:
		GeneralOptions& generalOptions() { return _generalOptions; }
		const GeneralOptions& generalOptions() const { return _generalOptions; }
		StartupOptions& startupOptions() { return _startupOptions; }
		const StartupOptions& startupOptions() const { return _startupOptions; }
		ImageOptions& imageOptions() { return _imageOptions; }
		const ImageOptions& imageOptions() const { return _imageOptions; }
		ImageEditorOptions& imageEditorOptions() { return _imageEditorOptions; }
		const ImageEditorOptions& imageEditorOptions() const { return _imageEditorOptions; }

		MRUStringList& recentProjects() { return _recentProjects; }
		const MRUStringList& recentProjects() const { return _recentProjects; }

	public:
		void getWindowState(QString name, QMainWindow* window);
		void setWindowState(QString name, const QMainWindow* window);

		void getSplitterState(QString name, QSplitter* splitter);
		void setSplitterState(QString name, const QSplitter* splitter);

		QString getFileDialogDir(QString dlgName);
		void setFileDialogDir(QString dlgName, QString dir);

	signals:
		void settingsChanged();

	private:
		void loadGeneralOptions();
		void saveGeneralOptions();
		void loadStartupOptions();
		void saveStartupOptions();
		void loadImageOptions();
		void saveImageOptions();
		void loadImageEditorOptions();
		void saveImageEditorOptions();

		void loadProjectFilesMRUList();
		void saveProjectFilesMRUList();

	private:
		GeneralOptions _generalOptions;
		StartupOptions _startupOptions;
		ImageOptions _imageOptions;
		ImageEditorOptions _imageEditorOptions;

		MRUStringList _recentProjects{false, true, 8};
	};
}

#endif
