/******************************************************************************
 * File: EngineExecutionContext.h
 * Date: 21.2.2018
 *****************************************************************************/

#ifndef ENGINEEXECUTIONCONTEXT_H
#define ENGINEEXECUTIONCONTEXT_H

#include <map>

#include "Engine.h"
#include "pipeline/BlockHierarchy.h"
#include "data/DataBlob.h"

namespace grndr
{
	class Engine;
	class Label;
	class Port;
	class ImageReference;

	class EngineExecutionContext
	{	
	public:
		EngineExecutionContext(const Engine* engine, Label* label, const ImageReference* activeImageReference, Engine::ExecutionMode mode);

	public:
		void purge(const BlockHierarchy& blockHierarchy, unsigned int reachedLevel);

	public:
		const Engine* engine() const { return _engine; }
		Label* label() { return _label; }
		const Label* label() const { return _label; }
		const ImageReference* activeImageReference() const { return _activeImageReference; }

		DataBlob* contextEntry(const Port* port);
		void setContextEntry(const Port* port, const DataBlob& data);
		void setContextEntry(const Port* port, DataBlob&& data);

		bool contains(const Port* port) const { return _contextEntries.find(port) != _contextEntries.end(); }

		Engine::ExecutionMode getExecutionMode() const { return _executionMode; }
		void setExecutionMode(Engine::ExecutionMode mode) { _executionMode = mode; }

		bool wasAborted() const { return _abortProcessing; }
		void abortProcessing() { _abortProcessing = true; }

	private:
		bool canPurge(const Port* port, const BlockHierarchy& blockHierarchy, unsigned int reachedLevel) const;

	private:
		const Engine* _engine{nullptr};
		Label* _label{nullptr};
		const ImageReference* _activeImageReference{nullptr};

		std::map<const Port*, DataBlob> _contextEntries;

		Engine::ExecutionMode _executionMode{Engine::ExecutionMode::Execute};
		bool _abortProcessing{false};
	};
}

#endif
