/******************************************************************************
 * File: DataBlob.impl.h
 * Date: 19.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DataBlob.h"

#include <limits>
#include <cstring>

template<typename DataType>
DataBlob::DataBlob(const DataDescriptor& dataDesc, const DataType& data) : DataBlob(dataDesc)
{
	set(data);
}

template<typename DataType>
DataBlob::DataBlob(const DataDescriptor& dataDesc, DataType&& data) : DataBlob(dataDesc)
{
	set(std::move(data));
}

template<typename DataType>
void DataBlob::set(const std::vector<DataType>& data)
{
	if (data.size() > 0)
	{
		// Create a matrix with a single row that holds the vector data
		_data.create(1, data.size(), _dataDescriptor.getCVMatrixType());

		// Perform a direct memory copy of the data
		DataType* matrixData = _data.ptr<DataType>(0);
		std::memcpy(matrixData, data.data(), data.size() * sizeof(DataType));
	}
	else
		clear();
}

template<typename DataType>
void DataBlob::set(const DataType& data)
{
	// Create a matrix with a single row and a single column that holds the value
	_data.create(1, 1, _dataDescriptor.getCVMatrixType());
	_data.at<DataType>(0, 0) = data;
}
template<typename DataType>
std::vector<DataType> DataBlob::getVector() const
{
	if (!_data.empty())
	{
		std::vector<DataType> vec(_data.cols);

		// Perform a direct memory copy of the data
		DataType* vecData = vec.data();
		std::memcpy(vecData, _data.ptr<DataType>(0), _data.cols * sizeof(DataType));

		return vec;
	}
	else
		return {};
}

template<typename DataType>
DataType DataBlob::getScalar() const
{
	if (!_data.empty())
		return _data.at<DataType>(0, 0);
	else
		return DataType{};
}

template<typename TargetType>
void DataBlob::convertTo(const DataDescriptor& dataDesc)
{
	return convertTo(dataDesc, true, std::numeric_limits<TargetType>::min(), std::numeric_limits<TargetType>::max());
}
