/******************************************************************************
 * File: DataDescriptor.h
 * Date: 15.2.2018
 *****************************************************************************/

#ifndef DATADESCRIPTOR_H
#define DATADESCRIPTOR_H

#include <QString>

namespace grndr
{
	class DataDescriptor
	{
	public:
		enum class StructureType
		{
			Any,	/* Arbitrary type */
			Adaptive,	/* Data adapts to its input */

			Scalar,
			Vector,
			Matrix,
		};

		enum class FieldType
		{
			Any,	/* Arbitrary type */
			Adaptive,	/* Data adapts to its input */

			Basic,
			Color,
			Point2D,
			Point3D,
		};

		enum class ValueType
		{
			Any,	/* Arbitrary type */
			Adaptive,	/* Data adapts to its input */

			Int8,
			UInt8,
			Int16,
			UInt16,
			Int32,
			UInt32,
			Float,
			Double,
		};

	public:
		static DataDescriptor arbitraryDescriptor(QString name = "Arbitrary data");
		static DataDescriptor imageDescriptor(bool colorImage = true, ValueType valueType = ValueType::UInt8);
		static DataDescriptor adaptiveOutputDescriptor(QString name, StructureType structType = StructureType::Matrix);

	public:
		DataDescriptor() { }
		DataDescriptor(QString name, StructureType structureType, FieldType fieldType, ValueType valueType);

	public:
		bool canConvertTo(const DataDescriptor& dataDesc) const;
		bool canConvertToColor(const DataDescriptor& dataDesc) const;
		bool canConvertToGrayscale(const DataDescriptor& dataDesc) const;

		bool match(const DataDescriptor& dataDesc, DataDescriptor* adjustedDataDesc = nullptr) const;

		int getCVMatrixType(int* chan = nullptr, int* dep = nullptr) const;

	public:
		QString getName() const { return _name; }

		StructureType getStructureType() const { return _structureType; }
		FieldType getFieldType() const { return _fieldType; }
		ValueType getValueType() const { return _valueType; }

		bool isArbitrary() const;
		bool isAdaptive() const;

	private:
		template<typename Type>
		bool canConvertTypes(const std::vector<std::pair<Type, Type>> validConversions, Type typeFrom, Type typeTo) const;

	private:
		QString _name{""};

		StructureType _structureType{StructureType::Any};
		FieldType _fieldType{FieldType::Any};
		ValueType _valueType{ValueType::Any};
	};

	using DataDescriptors = std::vector<DataDescriptor>;
}

#include "DataDescriptor.impl.h"

#endif
