/******************************************************************************
 * File: DraftItem.cpp
 * Date: 20.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DraftItem.h"
#include "Layer.h"

const char* DraftItem::Serialization_Value_Type = "Type";

DraftItem::DraftItem(Layer* layer, DraftItemType type) : LayerItem(layer),
	_type{type}
{
	if (!layer)
		throw std::invalid_argument{_EXCPT("layer may not be null")};
}

void DraftItem::initDraftItem()
{
	createProperties();
}

int DraftItem::getZOrder() const
{
	return _layer->getZOrder();
}

void DraftItem::serialize(SerializationContext& ctx) const
{
	PropertyObject::serialize(ctx);

	// Serialize values
	ctx.settings()[Serialization_Value_Type] = _type;
}

void DraftItem::deserialize(DeserializationContext& ctx)
{
	PropertyObject::deserialize(ctx);
}

void DraftItem::createProperties()
{
	PropertyObject::createProperties();

	// Create standard properties
	_primaryColor = createProperty<ColorProperty>(PropertyID::PrimaryColor, "Primary color", QColor{255, 255, 255}, PropertyBase::Flag::Hidden);
	primaryColor()->setDescription("The primary color to use.");

	_position = createProperty<PointProperty>(PropertyID::Position, "Position", QPoint{0, 0});
	position()->setDescription("The item position.");

	_hasDirection = createProperty<BoolProperty>(PropertyID::HasDirection, "Has direction", false);
	hasDirection()->setDescription("Specifies whether the box has a direction/orientation.");

	_direction = createProperty<AngleProperty>(PropertyID::Direction, "Direction", 0.0);
	direction()->createConstraint<RangeConstraint>(0, 360);
	direction()->setDescription("The direction/orientation of the box contents.");

	_imageTagsAllotment = createProperty<ImageTagsAllotmentProperty>(PropertyID::ImageTagsAllotment, "Tags");
	imageTagsAllotment()->setDescription("The assigned tags.");
	imageTagsAllotment()->object().setImageBuild(_layer->imageBuild());
}
