/******************************************************************************
 * File: DraftItemCatalog.cpp
 * Date: 20.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DraftItemCatalog.h"
#include "DraftItem.h"

#include "draftitems/LineDraftItem.h"
#include "draftitems/BoxDraftItem.h"
#include "draftitems/PixelsDraftItem.h"

#define REGISTER_ITEM_TYPE(cls)	registerDraftItemType(cls::type_value, [](Layer* layer) { return std::make_unique<cls>(layer); })

std::map<DraftItemType, DraftItemCatalog::item_creator_type> DraftItemCatalog::s_creators;

std::set<DraftItemType> DraftItemCatalog::getTypes()
{
	std::set<DraftItemType> types;

	for (const auto& creator : s_creators)
		types.insert(creator.first);

	return types;
}

void DraftItemCatalog::registerDraftItemType(DraftItemType type, item_creator_type creator)
{
	if (type == DraftItemType::Undefined)
		throw std::invalid_argument{_EXCPT("type may not be DraftItemType::Undefined")};

	if (!creator)
		throw std::invalid_argument{_EXCPT("creator may not be null")};

	s_creators[type] = creator;
}

std::unique_ptr<DraftItem> DraftItemCatalog::createDraftItem(Layer* layer, DraftItemType type)
{
	if (type == DraftItemType::Undefined)
		throw std::invalid_argument{_EXCPT("type may not be DraftItemType::Undefined")};

	if (s_creators.find(type) != s_creators.end())
	{
		auto creator = s_creators.at(type);
		auto item = creator(layer);

		if (!item)
			throw std::runtime_error{_EXCPT(QString{"Failed to create a draft item of type %1"}.arg(type))};

		return item;
	}
	else
		throw std::invalid_argument{_EXCPT("The given type has not been registered")};
}

void DraftItemCatalog::registerStandardDraftItems()
{
	REGISTER_ITEM_TYPE(LineDraftItem);
	REGISTER_ITEM_TYPE(BoxDraftItem);
	REGISTER_ITEM_TYPE(PixelsDraftItem);
}
