/******************************************************************************
 * File: DraftItemRenderer.h
 * Date: 21.3.2018
 *****************************************************************************/

#ifndef DRAFTITEMRENDERER_H
#define DRAFTITEMRENDERER_H

#include "ItemRenderer.h"

namespace grndr
{
	class DraftItem;

	template<typename ItemType>
	class DraftItemRenderer : public ItemRenderer
	{
		static_assert(std::is_base_of<DraftItem, ItemType>::value, "ItemType must be derived from DraftItem");

	public:
		using item_type = ItemType;

	public:
		DraftItemRenderer(const item_type* item, const RendererStyle& rendererStyle);

	protected:
		virtual QPoint getItemPosition(RenderMode mode) const override;

		QPen createStandardPen(int lineWidth, Qt::PenCapStyle capStyle = Qt::FlatCap) const;
		QPen createSelectionPen(int lineWidth, bool inactive = false) const;

	protected:
		void renderDecoration(QPainter* painter, QRect itemRect, RenderFlags flags);

		void renderDirectionArrow(QPainter* painter, QPoint centerPos);
		void renderImageTags(QPainter* painter, QPoint centerPos, QRect itemRect);

	protected:
		const item_type* _draftItem{nullptr};
	};
}

#include "DraftItemRenderer.impl.h"

#endif
