/******************************************************************************
 * File: ImageBuild.h
 * Date: 12.3.2018
 *****************************************************************************/

#ifndef IMAGEBUILD_H
#define IMAGEBUILD_H

#include <opencv2/core.hpp>

#include "common/properties/PropertyObject.h"
#include "LayerVector.h"
#include "properties/ImageTagsAllotmentProperty.h"

namespace grndr
{
	class ImageReference;
	class ImageTags;

	class ImageBuild : public PropertyObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_ImageReferences;

	public:
		ImageBuild(const Block* block, const std::vector<const ImageReference*>& imageReferences);

	public:
		void initImageBuild();

	public:
		std::shared_ptr<Layer> createLayer(QString name = "");
		void removeLayer(const Layer* layer);
		void removeAllLayers();

		void moveLayer(const Layer* layer, bool up);

	public:
		const Block* block() const { return _block; }
		const std::vector<const ImageReference*>& imageReferences() const { return _imageReferences; }

		const cv::Mat& imageData() const { return _imageData; }
		void setImageData(const cv::Mat& data) { _imageData = data; emit imageDataChanged(); }
		void clearImageData() { _imageData.release(); emit imageDataChanged(); }

		const LayerVector& layers() const { return _layers; }

		ImageTags* inputImageTags() const;

		auto imageTagsAllotment() { return dynamic_cast<ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }
		auto imageTagsAllotment() const { return dynamic_cast<const ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }

	public:
		void serialize(SerializationContext& ctx) const override;
		void deserialize(DeserializationContext& ctx) override;

	signals:
		void layerCreated(const std::shared_ptr<Layer>&);
		void layerRemoved(const std::shared_ptr<Layer>&);
		void layerMoved(const std::shared_ptr<Layer>&, int, int);

		void imageDataChanged();

		void inputImageTagsChanged(const ImageTags*) const;

	protected:
		virtual void createProperties() override;

	private slots:
		void imageTagsConnectionChanged() const { emit inputImageTagsChanged(inputImageTags()); }

	private:
		const Block* _block{nullptr};
		std::vector<const ImageReference*> _imageReferences;
		cv::Mat _imageData;

		LayerVector _layers;

		std::shared_ptr<PropertyBase> _imageTagsAllotment;
	};
}

#endif
