/******************************************************************************
 * File: ImageExceptions.h
 * Date: 12.3.2018
 *****************************************************************************/

#ifndef IMAGEEXCEPTIONS_H
#define IMAGEEXCEPTIONS_H

#include "core/GrinderExceptions.h"

namespace grndr
{
	class ImageBuild;
	class ImageEditor;
	class Layer;
	class ImageTag;

	class ImageException : public GrinderException
	{
	public:
		ImageException(QString what);
	};

	class ImageBuildException : public ImageException
	{
	public:
		ImageBuildException(const ImageBuild* imageBuild, QString what);

	public:
		const ImageBuild* imageBuild() const { return _imageBuild; }

	private:
		const ImageBuild* _imageBuild{nullptr};
	};

	class ImageEditorException : public ImageException
	{
	public:
		ImageEditorException(const ImageEditor* imageEditor, QString what);

	public:
		const ImageEditor* imageEditor() const { return _imageEditor; }

	private:
		const ImageEditor* _imageEditor{nullptr};
	};

	class LayerException : public ImageBuildException
	{
	public:
		LayerException(const Layer* layer, QString what);

	public:
		const Layer* imageEditor() const { return _layer; }

	private:
		const Layer* _layer{nullptr};
	};

	class ImageTagException : public ImageException
	{
	public:
		ImageTagException(const ImageTag* imageTag, QString what);

	public:
		const ImageTag* imageTag() const { return _imageTag; }

	private:
		const ImageTag* _imageTag{nullptr};
	};
}

#endif
