/******************************************************************************
 * File: ImageTags.h
 * Date: 17.4.2018
 *****************************************************************************/

#ifndef IMAGETAGS_H
#define IMAGETAGS_H

#include <QObject>

#include "ImageTagVector.h"

namespace grndr
{
	class ImageTags : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;
		static const char* Serialization_Element;

	public:
		ImageTags& operator =(const ImageTags& imageTags);

	public:
		void addTag(const std::shared_ptr<ImageTag>& tag);
		void updateTag(const ImageTag* tag);
		void removeTag(const ImageTag* tag);
		void removeAllTags();

		std::shared_ptr<ImageTag> makeTag(QString name) { return std::make_shared<ImageTag>(this, name, QColor{}); }

	public:
		const ImageTagVector& tags() const { return _tags; }

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void tagAdded(const std::shared_ptr<ImageTag>&);
		void tagRemoved(const std::shared_ptr<ImageTag>&);
		void tagModified(const std::shared_ptr<ImageTag>&);

	private:
		ImageTagVector _tags;
	};
}

#endif
