/******************************************************************************
 * File: Layer.h
 * Date: 17.3.2018
 *****************************************************************************/

#ifndef LAYER_H
#define LAYER_H

#include "ImageBuildItem.h"
#include "DraftItemVector.h"
#include "LayerPixels.h"

namespace grndr
{
	class Layer : public ImageBuildItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Visible;

	public:
		Layer(ImageBuild* imageBuild, QString name = "");

	public:
		void initLayer();

	public:
		std::shared_ptr<DraftItem> createDraftItem(DraftItemType type);
		void removeDraftItem(const DraftItem* item);

	public:
		int getZOrder() const;

		bool isVisible() const { return _isVisible; }
		void setVisible(bool visible = true);

		const DraftItemVector& draftItems() const { return _draftItems; }
		LayerPixels& layerPixels() { return _layerPixels; }
		const LayerPixels& layerPixels() const { return _layerPixels; }

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	signals:
		void layerShown();
		void layerHidden();

		void draftItemCreated(const std::shared_ptr<DraftItem>&);
		void draftItemRemoved(const std::shared_ptr<DraftItem>&);

	private:
		bool _isVisible{true};

		DraftItemVector _draftItems;
		LayerPixels _layerPixels;
	};
}

#endif
