/******************************************************************************
 * File: PixelsDraftItem.cpp
 * Date: 20.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "PixelsDraftItem.h"
#include "image/renderers/PixelsDraftItemRenderer.h"

const DraftItemType PixelsDraftItem::type_value = DraftItemType::Pixels;

PixelsDraftItem::PixelsDraftItem(Layer* layer) : DraftItem(layer, type_value)
{

}

std::unique_ptr<ItemRenderer> PixelsDraftItem::createRenderer(const ItemRenderer::RendererStyle& rendererStyle) const
{
	return std::make_unique<PixelsDraftItemRenderer>(this, rendererStyle);
}

void PixelsDraftItem::serialize(SerializationContext& ctx) const
{
	DraftItem::serialize(ctx);

	// Serialize pixels data
	ctx.beginGroup(LayerPixelsData::Serialization_Group);
	_pixelsData.serialize(ctx);
	ctx.endGroup();
}

void PixelsDraftItem::deserialize(DeserializationContext& ctx)
{
	DraftItem::deserialize(ctx);

	// Deserialize pixels data
	if (ctx.beginGroup(LayerPixelsData::Serialization_Group))
	{
		_pixelsData.deserialize(ctx);
		ctx.endGroup();
	}
}

void PixelsDraftItem::createProperties()
{
	DraftItem::createProperties();

	// Create specific properties
	_showBox = createProperty<BoolProperty>(PropertyID::ShowBox, "Show box", false);
	showBox()->setDescription("Show the bounding box enclosing the pixels.");

	_boxMargin = createProperty<UIntProperty>(PropertyID::Margin, "Box margin", 0);
	boxMargin()->createConstraint<RangeConstraint>(0, 255);
	boxMargin()->setDescription("The margin between the pixels and the bounding box.");

	_boxLineWidth = createProperty<UIntProperty>(PropertyID::Width, "Box width", 1);
	boxLineWidth()->createConstraint<RangeConstraint>(1, 100);
	boxLineWidth()->setDescription("The line width of the bounding box.");
}
