/******************************************************************************
 * File: LineDraftItemRenderer.cpp
 * Date: 21.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LineDraftItemRenderer.h"
#include "util/MathUtils.h"

void LineDraftItemRenderer::render(QPainter* painter, RenderMode mode, RenderFlags flags)
{
	painter->save();

	QPoint endPosition = *_draftItem->endPosition();

	if (mode.testFlag(RenderModeFlag::RelativePositions))
		endPosition -= *_draftItem->position();

	// Draw the selection background as an outer glow
	if (flags.testFlag(RenderFlag::Selected))
	{
		painter->save();
		painter->setPen(createSelectionPen(*_draftItem->lineWidth(), flags.testFlag(RenderFlag::Inactive)));
		painter->setOpacity(_rendererStyle.selectionOpacity);
		painter->drawLine(getItemPosition(mode), endPosition);
		painter->restore();
	}

	// Draw the line
	painter->setPen(createStandardPen(*_draftItem->lineWidth()));
	painter->drawLine(getItemPosition(mode), endPosition);

	// Draw the item's decoration (direction, tags etc.)
	renderDecoration(painter, shape(flags).boundingRect().toRect(), flags);

	painter->restore();
}

QPainterPath LineDraftItemRenderer::shape(RenderFlags flags) const
{
	Q_UNUSED(flags);

	// Create a shape that is slightly thicker than the shown line
	QPainterPath linePath;
	linePath.moveTo(QPointF{0.0, 0.0});
	linePath.lineTo(*_draftItem->endPosition() - *_draftItem->position());

	QPainterPathStroker ps{createSelectionPen(*_draftItem->lineWidth())};
	auto path = ps.createStroke(linePath);
	path.addPath(linePath);

	return path;
}
