/******************************************************************************
 * File: PixelsDraftItemRenderer.cpp
 * Date: 29.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "PixelsDraftItemRenderer.h"
#include "image/LayerPixelsData.h"

PixelsDraftItemRenderer::PixelsDraftItemRenderer(const PixelsDraftItem* item, const ItemRenderer::RendererStyle& rendererStyle) : DraftItemRenderer(item, rendererStyle),
	_pixelsRenderer{&item->pixelsData(), rendererStyle}
{

}

void PixelsDraftItemRenderer::render(QPainter* painter, ItemRenderer::RenderMode mode, ItemRenderer::RenderFlags flags)
{
	painter->save();

	// Get the bounding box of the pixels
	auto rect = getBoundingBox();
	rect.moveTo(getItemPosition(mode));	

	// Use the layer pixels renderer to render the pixels
	_pixelsRenderer.setPixelsOffset(getItemPosition(mode));
	_pixelsRenderer.render(painter, mode, flags);

	// Draw the selection background as an outer glow
	if (flags.testFlag(RenderFlag::Selected))
	{
		auto selectionPen = createSelectionPen(*_draftItem->boxLineWidth(), flags.testFlag(RenderFlag::Inactive));
		int margin = std::lround(selectionPen.widthF() / 2.0);

		painter->save();
		painter->setPen(selectionPen);
		painter->setOpacity(_rendererStyle.selectionOpacity);
		painter->drawRect(rect + QMargins{margin, margin, margin, margin});
		painter->restore();
	}

	// Draw the bounding box
	if (*_draftItem->showBox())
	{
		int margin = std::lround(*_draftItem->boxLineWidth() / 2.0);

		painter->setPen(createStandardPen(*_draftItem->boxLineWidth()));
		painter->drawRect(rect + QMargins{margin, margin, margin, margin});
	}

	// Draw the item's decoration (direction, tags etc.)
	renderDecoration(painter, rect, flags);

	painter->restore();
}

QPainterPath PixelsDraftItemRenderer::shape(RenderFlags flags) const
{
	if (*_draftItem->showBox() || flags.testFlag(RenderFlag::Selected))	// If the box is shown or the item is selected, return the bounding box as the item's shape
	{
		QPainterPath path;
		int margin = *_draftItem->boxLineWidth();

		path.addRect(getBoundingBox() + QMargins{margin, margin, margin, margin});
		return path;
	}
	else
		return _pixelsRenderer.shape(flags);
}

QRect PixelsDraftItemRenderer::getBoundingBox() const
{
	auto rect = _draftItem->pixelsData().bounds();	
	int margin = *_draftItem->boxMargin();
	rect += QMargins{margin, margin, margin - 1, margin - 1};
	return rect;
}
