/******************************************************************************
 * File: Block.h
 * Date: 15.1.2018
 *****************************************************************************/

#ifndef BLOCK_H
#define BLOCK_H

#include <set>

#include "PipelineItem.h"
#include "BlockType.h"
#include "BlockCategory.h"
#include "PortVector.h"

namespace grndr
{
	class ProcessorBase;

	class Block : public PipelineItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Type;
		static const char* Serialization_Value_Index;

	public:
		Block(Pipeline* pipeline, BlockType type, BlockCategory category, QString name = "");

	public:
		virtual void initBlock();

		virtual void verifyConnection(const Port* src, const Port* dest) const;

	public:
		virtual std::unique_ptr<ProcessorBase> createProcessor() const = 0;

	public:
		void walk(std::function<void(const Block*)> callback, Port::Direction direction) const;

		std::set<const ImageReference*> assembleInputImageReferences(const ImageReference* activeImageRef) const;

	public:
		const Port* dataPort(PortType portType) const;
		const Port* dataPort(const Port* port) const;

		template<typename PropType>
		PropType* portProperty(PortType portType, PropertyID propertyID) const;
		template<typename PropType>
		PropType* portProperty(const Port* port, PropertyID propertyID) const;

	public:
		QString getFormattedName() const;
		BlockType getType() const { return _type; }
		BlockCategory getCategory() const { return _category; }

		const PortVector& ports() const { return _ports; }

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	protected:
		virtual void createPorts() = 0;

		std::shared_ptr<Port> createPort(PortType type, Port::Direction dir, DataDescriptors dataDescriptors, QString name = "");
		void removePort(PortType type);
		void removePort(const Port* port);

	protected:
		BlockType _type{BlockType::Undefined};
		BlockCategory _category{BlockCategory::Undefined};

		PortVector _ports;
	};
}

#include "Block.impl.h"

#endif
