/******************************************************************************
 * File: BlockHierarchy.h
 * Date: 20.2.2018
 *****************************************************************************/

#ifndef BLOCKHIERARCHY_H
#define BLOCKHIERARCHY_H

#include <vector>

namespace grndr
{
	class Pipeline;
	class Block;

	class BlockHierarchy
	{
	public:
		using HierarchyLevel = std::vector<const Block*>;

	public:
		BlockHierarchy(const Pipeline* pipeline = nullptr);

	public:
		void createFromPipeline(const Pipeline* pipeline = nullptr);

	public:
		auto at(int index) { return _hierarchy.at(index); }
		auto at(int index) const { return _hierarchy.at(index); }

		auto size() const { return _hierarchy.size(); }
		auto empty() const { return _hierarchy.empty(); }

		auto begin() { return _hierarchy.begin(); }
		auto begin() const { return _hierarchy.begin(); }
		auto cbegin() const { return _hierarchy.cbegin(); }
		auto end() { return _hierarchy.end(); }
		auto end() const { return _hierarchy.end(); }
		auto cend() const { return _hierarchy.cend(); }

		int getBlockLevel(const Block* block) const;

	private:
		void createNextLevel(const Pipeline* pipeline);

		bool isBlockInHierarchy(const Block* block) const;

		HierarchyLevel findLevel0Blocks(const Pipeline* pipeline) const;
		HierarchyLevel findNextLevelBlocks(const Pipeline* pipeline, const HierarchyLevel& level) const;
		HierarchyLevel findNextLevelBlocks(const Block* block) const;

		bool checkBlockInputs(const Block* block, const Block* currentBlock, const HierarchyLevel& currentLevel) const;

	private:
		std::vector<HierarchyLevel> _hierarchy;
	};
}

#endif
