/******************************************************************************
 * File: Connection.cpp
 * Date: 15.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "Connection.h"
#include "Port.h"

const char* Connection::Serialization_Value_Source = "Source";
const char* Connection::Serialization_Value_Dest = "Dest";

Connection::Connection(Port* src, Port* dest) : PipelineItem(src->pipeline()),
	_sourcePort{src}, _destPort{dest}
{
	if (!src)
		throw std::invalid_argument{_EXCPT("src may not be null")};

	if (!dest)
		throw std::invalid_argument{_EXCPT("dest may not be null")};
}

void Connection::initConnection()
{
	PipelineItem::initPipelineItem();
}

Block* Connection::_source() const
{
	if (_sourcePort)
		return _sourcePort->block();
	else
		return nullptr;
}

Block* Connection::_dest() const
{
	if (_destPort)
		return _destPort->block();
	else
		return nullptr;
}

void Connection::serialize(SerializationContext& ctx) const
{
	PipelineItem::serialize(ctx);

	// Serialize values
	ctx.settings()[Serialization_Value_Source] = ctx.getPortIndex(_sourcePort);
	ctx.settings()[Serialization_Value_Dest] = ctx.getPortIndex(_destPort);
}

void Connection::deserialize(DeserializationContext& ctx)
{
	PipelineItem::deserialize(ctx);

	int sourceIndex = ctx.settings()[Serialization_Value_Source].toInt();
	int destIndex = ctx.settings()[Serialization_Value_Dest].toInt();
	auto sourcePort = ctx.getPort(sourceIndex);
	auto destPort = ctx.getPort(destIndex);

	if (sourcePort && destPort)
	{
		_sourcePort = sourcePort;
		_destPort = destPort;
	}
}
