/******************************************************************************
 * File: Pipeline.h
 * Date: 15.1.2018
 *****************************************************************************/

#ifndef PIPELINE_H
#define PIPELINE_H

#include "BlockVector.h"

namespace grndr
{
	class PipelineManager;

	class Pipeline : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Name;

	public:
		Pipeline(QString name = "");

	public:
		void initPipeline();

	public:
		std::shared_ptr<Block> createBlock(BlockType type, QString name = "");
		void removeBlock(const Block* block);

	public:
		QString getName() const { return _name; }
		void setName(QString name) { if (name != _name) { _name = name; emit pipelineRenamed(); } }

		const BlockVector& blocks() const { return _blocks; }		

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void pipelineRenamed();

		void blockCreated(const std::shared_ptr<Block>&);
		void blockRemoved(const std::shared_ptr<Block>&);
		void connectionCreated(const std::shared_ptr<Connection>&);
		void connectionRemoved(const std::shared_ptr<Connection>&);

	private:
		QString _name{""};

		BlockVector _blocks;
	};
}

#endif
