/******************************************************************************
 * File: ImageTagsBlock.cpp
 * Date: 03.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageTagsBlock.h"
#include "core/GrinderApplication.h"
#include "pipeline/PipelineExceptions.h"
#include "engine/processors/InputProcessor.h"

const BlockType ImageTagsBlock::type_value = BlockType::ImageTags;
const BlockCategory ImageTagsBlock::category_value = BlockCategory::Annotation;

ImageTagsBlock::ImageTagsBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

void ImageTagsBlock::verifyConnection(const Port* src, const Port* dest) const
{
	Block::verifyConnection(src, dest);

	// Check if the tags-out port is connected to a tags-in port
	if (src == _imageTagsPort.get() && dest->getType() != PortType::ImageTagsIn)
		throw BlockException{this, _EXCPT("The tags-out port must be connected to a tags-in port")};
}

std::unique_ptr<ProcessorBase> ImageTagsBlock::createProcessor() const
{
	// This block has no processor
	return nullptr;
}

void ImageTagsBlock::createProperties()
{
	Block::createProperties();

	_imageTags = createProperty<ImageTagsProperty>(PropertyID::ImageTags, "Tags");
	imageTags()->setDescription("A list of user-definable tags.");
}

void ImageTagsBlock::createPorts()
{
	_imageTagsPort = createPort(PortType::ImageTagsOut, Port::Direction::Out, {}, "Tags");
}
