/******************************************************************************
 * File: ImageReference.h
 * Date: 10.2.2018
 *****************************************************************************/

#ifndef IMAGEREFERENCE_H
#define IMAGEREFERENCE_H

#include <QDateTime>
#include <QSize>
#include <opencv2/core.hpp>

#include "project/ProjectItem.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class Project;

	class ImageReference : public ProjectItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_File;
		static const char* Serialization_Value_Index;

	public:
		struct ImageInfo
		{
			qint64 fileSize{0};
			QDateTime fileDate;

			QSize imageSize;
		};

	public:
		ImageReference(Project* project, QString imageFilePath, const QStringList& imageSearchPaths = QStringList{});

		bool operator ==(const ImageReference& ref) const { return *this == ref._imageFilePath; }
		bool operator ==(QString image) const;
		bool operator <(const ImageReference& ref) const { return *this < ref._imageFilePath; }
		bool operator <(QString image) const;

	public:
		cv::Mat loadImage() const;

		void refreshImageInfo();

	public:
		QString getImageFilePath() const { return _imageFilePath; }
		QString getImageFileName() const;
		ImageInfo getImageInfo() const { return _imageInfo; }

		bool isValid() const { return _isValid; }

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	private:
		void searchImage(const QStringList& imageSearchPaths);

	private:
		QString _imageFilePath{""};
		ImageInfo _imageInfo;

		bool _isValid{false};
	};
}

#endif
