/******************************************************************************
 * File: Label.cpp
 * Date: 07.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "Label.h"

const char* Label::Serialization_Group_Pipeline = "Pipeline";
const char* Label::Serialization_Group_Layout = "Layout";
const char* Label::Serialization_Group_ImageBuildPool = "ImageBuildPool";

const char* Label::Serialization_Value_Name = "Name";

Label::Label(Project* project, const std::shared_ptr<Pipeline>& pipeline) : ProjectItem(project),
	_pipeline{pipeline}, _imageBuildPool{this}
{
	if (!pipeline)
		throw std::invalid_argument{_EXCPT("pipeline may not be null")};
}

void Label::serialize(SerializationContext& ctx) const
{
	// Serialize values
	ctx.settings()[Serialization_Value_Name] = getName();

	// Serialize the pipeline
	ctx.beginGroup(Serialization_Group_Pipeline);
	_pipeline->serialize(ctx);
	ctx.endGroup();

	// Serialize the graph layout
	ctx.beginGroup(Serialization_Group_Layout, true);
	_graphLayout.serialize(ctx);
	ctx.endGroup();

	// Serialize the image builds
	ctx.beginGroup(Serialization_Group_ImageBuildPool, true);
	_imageBuildPool.serialize(ctx);
	ctx.endGroup();
}

void Label::deserialize(DeserializationContext& ctx)
{
	// Deserialize the pipeline
	if (ctx.beginGroup(Serialization_Group_Pipeline))
	{
		_pipeline->deserialize(ctx);
		ctx.endGroup();
	}

	// Deserialize the graph layout
	if (ctx.beginGroup(Serialization_Group_Layout))
	{
		_graphLayout.deserialize(_pipeline.get(), ctx);
		ctx.endGroup();
	}

	// Deserialize the image builds
	if (ctx.beginGroup(Serialization_Group_ImageBuildPool))
	{
		_imageBuildPool.deserialize(ctx);
		ctx.endGroup();
	}
}
