/******************************************************************************
 * File: Label.h
 * Date: 07.2.2018
 *****************************************************************************/

#ifndef LABEL_H
#define LABEL_H

#include <memory>

#include "project/ProjectItem.h"
#include "pipeline/Pipeline.h"
#include "image/ImageBuildPool.h"
#include "ui/graph/GraphLayout.h"

namespace grndr
{
	class Project;

	class Label : public ProjectItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group_Pipeline;
		static const char* Serialization_Group_Layout;
		static const char* Serialization_Group_ImageBuildPool;

		static const char* Serialization_Value_Name;

	public:
		Label(Project* project, const std::shared_ptr<Pipeline>& pipeline);

	public:
		Pipeline* pipeline() { return _pipeline.get(); }
		const Pipeline* pipeline() const { return _pipeline.get(); }
		GraphLayout& graphLayout() { return _graphLayout; }
		const GraphLayout& graphLayout() const { return _graphLayout; }

		ImageBuildPool& imageBuildPool() { return _imageBuildPool; }
		const ImageBuildPool& imageBuildPool() const { return _imageBuildPool; }

		QString getName() const { return _pipeline->getName(); }
		void setName(QString name) { _pipeline->setName(name); }

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	private:
		std::shared_ptr<Pipeline> _pipeline;
		GraphLayout _graphLayout;

		ImageBuildPool _imageBuildPool;
	};
}

#endif
