/******************************************************************************
 * File: Project.h
 * Date: 07.2.2018
 *****************************************************************************/

#ifndef PROJECT_H
#define PROJECT_H

#include <QObject>

#include "LabelVector.h"
#include "ImageReferenceVector.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class Project : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;
		static const char* Serialization_Value_Name;

	public:
		std::shared_ptr<Label> createLabel(QString name);
		void removeLabel(QString name);
		void removeLabel(const Label* label);

		std::shared_ptr<ImageReference> createImageReference(QString imagePath, const QStringList& imageSearchPaths = QStringList{});
		void removeImageReference(QString imagePath);
		void removeImageReference(const ImageReference* imageRef);

	public:
		QString name() const { return _name; }
		void setName(QString name) { _name = name; }

		const LabelVector& labels() const { return _labels; }
		const ImageReferenceVector& imageReferences() const { return _imageReferences; }

	public:
		void clear();

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void labelCreated(const std::shared_ptr<Label>&);
		void labelRemoved(const std::shared_ptr<Label>&);

		void imageReferenceCreated(const std::shared_ptr<ImageReference>&);
		void imageReferenceRemoved(const std::shared_ptr<ImageReference>&);

	private:
		QString _name{""};

		LabelVector _labels;
		ImageReferenceVector _imageReferences;
	};
}

#endif
