/******************************************************************************
 * File: ImageReferenceProperty.h
 * Date: 12.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageReferenceProperty.h"
#include "core/GrinderApplication.h"
#include "ui/properties/editors/ImageReferencePropertyEditor.h"

void ImageReferenceProperty::initProperty()
{
	property_type::initProperty();

	// Listen for removed image references
	connect(&grinder()->project(), &Project::imageReferenceRemoved, this, &ImageReferenceProperty::imageReferenceRemoved);
}

QWidget* ImageReferenceProperty::createEditor(QWidget* parent)
{
	return new ImageReferencePropertyEditor{this, parent};
}

QString ImageReferenceProperty::toString() const
{
	// Get the name of the selected image reference (if any)
	if (_value)
		return _value->getImageFileName();
	else
		return "<Active image>";
}

const ImageReference* ImageReferenceProperty::resolveImageReference(const ImageReference* activeImageReference) const
{
	const ImageReference* currentImageRef = *this;

	// Make sure that the image reference really exists (just in case)
	if (currentImageRef)
	{
		if (!std::any_of(grinder()->project().imageReferences().cbegin(), grinder()->project().imageReferences().cend(), [currentImageRef](const auto& imageRef) { return imageRef.get() == currentImageRef; }))
			currentImageRef = nullptr;
	}

	// If no image reference has been set, use the currently active one
	if (!currentImageRef)
		currentImageRef = activeImageReference ? activeImageReference : grinder()->projectController().activeImageReference();

	return currentImageRef;
}

void ImageReferenceProperty::serialize(SerializationContext& ctx) const
{
	// Override the default behavior to store the serialized image reference index
	PropertyBase::serialize(ctx);

	// Serialize values
	if (!hasFlag(Flag::ReadOnly))
		ctx.settings()[Serialization_Value_Value] = _value ? ctx.getImageReferenceIndex(_value) : -1;
}

void ImageReferenceProperty::deserialize(DeserializationContext& ctx)
{
	// Restore the image reference by using the stored image reference index
	PropertyBase::deserialize(ctx);

	// Deserialize values
	if (!hasFlag(Flag::ReadOnly))
	{
		auto imageRefIndex = ctx.settings()[Serialization_Value_Value].toInt();
		setValue(imageRefIndex != -1 ? ctx.getImageReference(imageRefIndex) : nullptr);
	}
}

void ImageReferenceProperty::imageReferenceRemoved(const std::shared_ptr<ImageReference>& imageReference)
{
	// If the currently set image reference is removed, unset this property
	if (imageReference.get() == _value)
		setValue(nullptr);
}
