/******************************************************************************
 * File: CheckListDialog.cpp
 * Date: 25.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "CheckListDialog.h"
#include "ui_CheckListDialog.h"

CheckListDialog::CheckListDialog(QString dialogTitle, bool acceptIfNoItemsChecked, QWidget *parent) : QDialog(parent, Qt::Dialog|Qt::WindowTitleHint|Qt::WindowCloseButtonHint),
	ui{new Ui::CheckListDialog}, _acceptIfNoItemsChecked{acceptIfNoItemsChecked}
{
	ui->setupUi(this);

	if (!dialogTitle.isEmpty())
		setWindowTitle(dialogTitle);
}

CheckListDialog::~CheckListDialog()
{
	delete ui;
}

void CheckListDialog::setupUi()
{
	ui->setupUi(this);
}

void CheckListDialog::setCheckListWidget(QListWidget* listWidget)
{
	if (!listWidget)
		throw std::invalid_argument{_EXCPT("listWidget may not be null")};

	_checkListWidget = listWidget;

	ui->verticalLayout->insertWidget(0, _checkListWidget);

	_checkListWidget->setSelectionMode(QListWidget::ExtendedSelection);
	_checkListWidget->setSelectionRectVisible(true);
	_checkListWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	_checkListWidget->setContextMenuPolicy(Qt::DefaultContextMenu);
	_checkListWidget->setFocus();

	// The OK button should only be enabled if at least one item is checked
	connect(_checkListWidget, &QListWidget::itemChanged, this, &CheckListDialog::checkListItemChanged);
}

void CheckListDialog::checkListItemChanged(QListWidgetItem* item)
{
	Q_UNUSED(item);

	bool anyItemChecked = false;

	if (!_acceptIfNoItemsChecked)
	{
		for (int i = 0; i < _checkListWidget->count(); ++i)
		{
			if (_checkListWidget->item(i)->checkState() == Qt::Checked)
			{
				anyItemChecked = true;
				break;
			}
		}
	}

	ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(anyItemChecked || _acceptIfNoItemsChecked);
}
