/******************************************************************************
 * File: GraphBlockNode.h
 * Date: 22.1.2018
 *****************************************************************************/

#ifndef GRAPHBLOCKNODE_H
#define GRAPHBLOCKNODE_H

#include <memory>
#include <QLineEdit>
#include <QGraphicsProxyWidget>

#include "GraphNode.h"
#include "GraphStyle.h"
#include "GraphConnectionBlueprint.h"

namespace grndr
{
	class Block;
	class GraphPortNode;
	class GraphConnectionBlueprint;

	class GraphBlockNode : public GraphNode
	{
		Q_OBJECT

		friend class GraphLayout;

	public:
		GraphBlockNode(GraphScene* scene, const std::shared_ptr<Block>& block, QGraphicsItem* parent = nullptr);

	public:		
		std::weak_ptr<Block>& block() { return _block; }
		const std::weak_ptr<Block>& block() const { return _block; }

		const std::vector<GraphPortNode*>& inPortNodes() const { return _inPortNodes; }
		const std::vector<GraphPortNode*>& outPortNodes() const { return _outPortNodes; }

	public:
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

	public slots:
		void beginRenameBlock();
		void endRenameBlock();
		void cancelRenameBlock();

	protected:
		virtual void keyReleaseEvent(QKeyEvent* event) override;

		virtual void mousePressEvent(QGraphicsSceneMouseEvent* event) override;
		virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* event) override;
		virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event) override;
		virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event) override;

	protected:
		virtual void updateGeometry() override;

		virtual std::vector<QAction*> getNodeActions(QMenu& menu) const override;

	private:
		QSizeF calcHeaderNameSize(const Block* block) const;
		QSizeF calcHeaderTypeSize(const Block* block) const;
		QSizeF calcPortsSize(const std::vector<GraphPortNode*>& ports) const;

		struct
		{
			QRectF headerRect;
			QSizeF headerNameSize;
			QSizeF headerTypeSize;

			QRectF portsRect;
			QSizeF inPortsSize;
			QSizeF outPortsSize;
		} _geometry;

	private:
		void drawHeader(QPainter* painter, const Block* block);

	private:
		void createPorts();
		void layoutPorts(const std::vector<GraphPortNode*>& ports, bool alignRight);

		GraphPortNode* findPortByPosition(QPointF pos);

	protected:
		std::weak_ptr<Block> _block;

		const GraphStyle::BlockNodeStyle& _blockStyle;

		std::vector<GraphPortNode*> _inPortNodes;
		std::vector<GraphPortNode*> _outPortNodes;

	private:
		QLineEdit* _nameEdit{nullptr};
		QGraphicsProxyWidget* _nameEditWidget{nullptr};
		bool _isEditingName{false};

		QAction* _renameAction;

	private:
		void beginConnectionInitiation(GraphPortNode* portNode);
		void updateConnectionInitiation(QPointF mousePos);
		void endConnectionInitiation(bool createConnection = true);

		std::unique_ptr<GraphConnectionBlueprint> _connectionBlueprint;
	};
}

#endif
