/******************************************************************************
 * File: GraphConnectionBlueprint.h
 * Date: 27.1.2018
 *****************************************************************************/

#ifndef GRAPHCONNECTIONBLUEPRINT_H
#define GRAPHCONNECTIONBLUEPRINT_H

#include "GraphConnectionBase.h"
#include "GraphConnectionMessage.h"
#include "pipeline/Port.h"

namespace grndr
{
	class GraphPortNode;

	class GraphConnectionBlueprint : public GraphConnectionBase
	{
	public:
		enum class Status
		{
			Pending,
			Valid,
			Invalid,
		};

	public:
		GraphConnectionBlueprint(GraphScene* scene, GraphPortNode* startPortNode, QGraphicsItem* parent = nullptr);

	public:
		void setTargetPortNode(GraphPortNode* node);
		void setTargetPos(QPointF pos);

		GraphPortNode* startPortNode() { return _startPortNode; }
		const GraphPortNode* startPortNode() const { return _startPortNode; }
		GraphPortNode* targetPortNode() { return  _targetPortNode; }
		const GraphPortNode* targetPortNode() const { return  _targetPortNode; }

		Port* connectionPort(Port::Direction dir) { return _connectionPort(dir); }
		const Port* connectionPort(Port::Direction dir) const { return _connectionPort(dir); }

	public:
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

	protected:
		virtual QPointF calcSourcePosition() const override;
		virtual QPointF calcDestPosition() const override;

	private:
		void validateConnection();

	private:
		Port* _connectionPort(Port::Direction dir) const;

	private:				
		GraphPortNode* _startPortNode{nullptr};
		GraphPortNode* _targetPortNode{nullptr};

		QPointF _targetPos;

		Status _status{Status::Pending};
		QString _statusMessage{""};
		GraphConnectionMessage* _statusMessageNode{nullptr};
	};
}

#endif
