/******************************************************************************
 * File: GraphLayout.h
 * Date: 23.2.2018
 *****************************************************************************/

#ifndef GRAPHLAYOUT_H
#define GRAPHLAYOUT_H

#include <QPointF>
#include <QSizeF>
#include <memory>

#include "pipeline/BlockHierarchy.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class BlockHierarchy;
	class GraphScene;

	class GraphLayout
	{
	public:
		static const char* Serialization_Group_Item;

		static const char* Serialization_Value_Block;
		static const char* Serialization_Value_Position;

	public:
		struct LayoutItem
		{
			LayoutItem() { }
			LayoutItem(std::weak_ptr<Block> block, QPointF pos) : _block{block}, _position{pos} { }

			std::weak_ptr<Block> _block;
			QPointF _position;

			void serialize(SerializationContext& ctx) const;
			void deserialize(DeserializationContext& ctx);
		};

		using Layout = std::vector<std::shared_ptr<LayoutItem>>;

	public:
		GraphLayout() { }
		GraphLayout(const GraphLayout& graphLayout) = default;
		GraphLayout(GraphLayout&& graphLayout) = default;
		GraphLayout(const Layout& layout) { _layout = layout; }
		GraphLayout(Layout&& layout) { _layout = std::move(layout); }

		GraphLayout& operator =(const GraphLayout& graphLayout) = default;
		GraphLayout& operator =(GraphLayout&& graphLayout) = default;
		GraphLayout& operator =(const Layout& layout) { _layout = layout; return *this; }
		GraphLayout& operator =(Layout&& layout) { _layout = std::move(layout); return *this; }

	public:
		void applyLayout(GraphScene* scene, bool centerLayout = true) const;

	public:
		void fromScene(GraphScene* scene);
		void fromBlockHierarchy(GraphScene* scene, const BlockHierarchy& blockHierarchy);

	public:
		Layout& layout() { return _layout; }
		const Layout& layout() const { return _layout; }

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(const Pipeline* pipeline, DeserializationContext& ctx);

	private:
		QSizeF addHierarchyLevel(GraphScene* scene, QPointF startPos, const BlockHierarchy::HierarchyLevel& blocks);

	private:
		Layout _layout;
	};
}

#endif
