/******************************************************************************
 * File: GraphPortNode.h
 * Date: 24.1.2018
 *****************************************************************************/

#ifndef GRAPHPORTNODE_H
#define GRAPHPORTNODE_H

#include <memory>

#include "GraphNode.h"
#include "GraphStyle.h"

namespace grndr
{
	class GraphBlockNode;
	class GraphConnectionBlueprint;
	class Port;

	class GraphPortNode : public GraphNode
	{
		Q_OBJECT

	public:
		GraphPortNode(GraphBlockNode* blockNode, GraphScene* scene, const std::shared_ptr<Port>& port, QGraphicsItem* parent = nullptr);

	public:
		GraphBlockNode* blockNode() { return _blockNode; }
		const GraphBlockNode* blockNode() const { return _blockNode; }
		std::weak_ptr<Port>& port() { return _port; }
		const std::weak_ptr<Port>& port() const { return _port; }

		QRectF getPortRect() const { return _geometry.portRect; }		
		QPointF getCenterPos(bool shiftCenter = true) const;

	public:
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

	protected:
		virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent* event) override;

	protected:
		virtual void updateGeometry() override;

		virtual std::vector<QAction*> getNodeActions(QMenu& menu) const override;

	private:
		QSizeF calcPortNameSize(const Port* port) const;

		struct
		{
			QRectF portRect;
			QSizeF portNameSize;
			QRectF portNameRect;
		} _geometry;

	private slots:
		void viewImage();
		void disconnectFromAll();

	private:
		void updateToolTip();

	private:
		GraphBlockNode* _blockNode{nullptr};
		std::weak_ptr<Port> _port;

		GraphStyle::PortNodeStyle _portStyle;

		QAction* _viewImageAction;
		QAction* _disconnectAction;
	};
}

#endif
