/******************************************************************************
 * File: GraphScene.h
 * Date: 18.1.2018
 *****************************************************************************/

#ifndef GRAPHSCENE_H
#define GRAPHSCENE_H

#include "ui/visscene/VisualScene.h"
#include "GraphView.h"
#include "GraphNodeFactory.h"

namespace grndr
{
	class Pipeline;
	class Block;
	class Port;
	class Connection;
	class GraphView;
	class GraphBlockNode;
	class GraphPortNode;
	class GraphConnectionNode;

	class GraphScene : public VisualScene<GraphView>
	{
		Q_OBJECT

	public:
		GraphScene(Pipeline* pipeline, GraphView* view, QSizeF initialSize);

	public:
		void buildScene();

		void createBlockNode(const std::shared_ptr<Block>& block);
		void removeBlockNode(const std::shared_ptr<Block>& block) { removeBlockNode(findBlockNode(block.get())); }
		void removeBlockNode(GraphBlockNode* blockNode);	

		void createConnectionNode(const std::shared_ptr<Connection>& connection);
		void removeConnectionNode(const std::shared_ptr<Connection>& connection) { removeConnectionNode(findConnectionNode(connection.get())); }
		void removeConnectionNode(GraphConnectionNode* connectionNode);

	public:
		GraphBlockNode* findBlockNode(const Block* block) { return _findBlockNode(block); }
		const GraphBlockNode* findBlockNode(const Block* block) const { return _findBlockNode(block); }
		GraphPortNode* findPortNode(const Port* port) { return _findPortNode(port); }
		const GraphPortNode* findPortNode(const Port* port) const { return _findPortNode(port); }
		GraphConnectionNode* findConnectionNode(const Connection* con) { return _findConnectionNode(con); }
		const GraphConnectionNode* findConnectionNode(const Connection* con) const { return _findConnectionNode(con); }

	public:
		const GraphNodeFactory& nodeFactory() const { return _nodeFactory; }

	private:
		GraphBlockNode* _findBlockNode(const Block* block) const;
		GraphPortNode* _findPortNode(const Port* port) const;
		GraphConnectionNode* _findConnectionNode(const Connection* con) const;

	private:
		Pipeline* _pipeline{nullptr};

		GraphNodeFactory _nodeFactory;
	};
}

#endif
