/******************************************************************************
 * File: GraphView.h
 * Date: 18.1.2018
 *****************************************************************************/

#ifndef GRAPHVIEW_H
#define GRAPHVIEW_H

#include <QToolBar>

#include "ui/visscene/VisualSceneView.h"
#include "GraphStyle.h"

namespace grndr
{
	class Label;

	class GraphView : public VisualSceneView
	{
		Q_OBJECT

	public:
		GraphView(QWidget* parent = nullptr);

	public:
		GraphScene* graphScene() { return _graphScene; }
		const GraphScene* graphScene() const { return _graphScene; }
		void setGraphScene(GraphScene* scene);

	public slots:
		void layoutGraph();	
		void updateCurrentGraphLayout() const;

	public slots:
		virtual void moveSelectedItems(QPoint delta) override;
		virtual void removeSelectedItems() const override;

	public:
		virtual const GraphStyle& sceneStyle() const override { return _graphStyle; }

	protected:		
		virtual void dragEnterEvent(QDragEnterEvent*event) override;
		virtual void dragMoveEvent(QDragMoveEvent* event) override;
		virtual void dropEvent(QDropEvent* event) override;

	protected:
		virtual std::vector<QAction*> getActions(AddActionsMode mode) const override;

	protected slots:
		virtual void updateActions() override;

	private slots:
		void projectSaving(QString fileName);

		void labelSwitching(Label* label);
		void labelSwitched(Label* label);

	private:
		GraphScene* _graphScene{nullptr};

		GraphStyle _graphStyle;

	private:
		QAction* _layoutGraphAction{nullptr};
	};
}

#endif
