/******************************************************************************
 * File: ColorPresetsWidget.cpp
 * Date: 03.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ColorPresetsWidget.h"
#include "util/UIUtils.h"

#define PRESET_COUNT	10

ColorPresetsWidget::ColorPresetsWidget(QWidget* parent) : QWidget(parent),
	_layout{new QGridLayout{this}}
{
	_layout->setContentsMargins(0, 0, 0, 0);
	_layout->setHorizontalSpacing(2);
	_layout->setVerticalSpacing(2);

	// Create all color widgets
	createPresetWidgets();
}

void ColorPresetsWidget::assignUiComponents(QWidget* actionOwner)
{
	// Create all actions
	for (unsigned int i = 0; i < _colorPresets.size(); ++i)
	{
		auto& presetEntry = _colorPresets[i];
		presetEntry.presetAction = createAction(i, actionOwner);
	}
}

void ColorPresetsWidget::createPresetWidgets()
{
	int row = 0;
	int col = 0;

	for (int i = 0; i < PRESET_COUNT; ++i)
	{
		auto colorWidget = new ColorWidget{ColorWidget::Flag::SmallBox|ColorWidget::Flag::SelectColorOnDoubleClick};
		colorWidget->setExtraToolTip(QString{"<b>Preset %1</b> (%2)"}.arg(i + 1).arg((i + 1) % 10));
		_colorPresets.push_back({colorWidget, nullptr});

		// React to color selections
		connect(colorWidget, &ColorWidget::colorClicked, this, &ColorPresetsWidget::presetSelected);

		_layout->addWidget(colorWidget, row, col++);

		if (col >= (PRESET_COUNT / 2))
		{
			row++;
			col = 0;
		}
	}
}

QAction* ColorPresetsWidget::createAction(unsigned int index, QWidget* owner)
{
	auto action = UIUtils::createAction(owner, QString{"Preset %1"}.arg(index + 1), "", SLOT(presetActionTriggered()), "", QString{"%1"}.arg((index + 1) % 10), Qt::WidgetWithChildrenShortcut, this);
	action->setData(index);
	return action;
}

void ColorPresetsWidget::presetSelected(QColor color)
{
	emit colorSelected(color);
}

void ColorPresetsWidget::presetActionTriggered()
{
	if (auto action = dynamic_cast<QAction*>(sender()))
	{
		unsigned int index = action->data().toUInt();
		emit colorSelected(getColor(index));
	}
}
