/******************************************************************************
 * File: ImageEditorEnvironment.h
 * Date: 26.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORENVIRONMENT_H
#define IMAGEEDITORENVIRONMENT_H

#include <QColor>

#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class ImageEditorController;

	class ImageEditorEnvironment : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;

		static const char* Serialization_Value_PrimaryColor;
		static const char* Serialization_Value_ShowDirectionArrows;
		static const char* Serialization_Value_ShowTags;

	public:
		ImageEditorEnvironment(ImageEditorController* controller);

	public:
		QColor getPrimaryColor() const { return _primaryColor; }
		void setPrimaryColor(QColor color);

		bool showDirectionArrows() const { return _showDirectionArrows; }
		void setShowDirectionArrows(bool show);
		bool showTags() const { return _showTags; }
		void setShowTags(bool show);

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void primaryColorChanged(QColor);
		void showDirectionArrowsChanged(bool);
		void showTagsChanged(bool);

	private:
		ImageEditorController* _editorController{nullptr};

	private:
		QColor _primaryColor{255, 255, 255};

		bool _showDirectionArrows{true};
		bool _showTags{true};
	};
}

#endif
