/******************************************************************************
 * File: ImageEditorNode.cpp
 * Date: 21.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageEditorNode.h"
#include "ImageEditorScene.h"
#include "ImageEditorView.h"
#include "ImageEditor.h"

ImageEditorNode::ImageEditorNode(ImageEditorScene* scene, QGraphicsItem* parent) : VisualNode(scene, parent), ImageEditorComponent(scene->imageEditor()),
	_scene{scene}, _style{scene->view()->sceneStyle()}
{

}

void ImageEditorNode::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	Q_UNUSED(option);
	Q_UNUSED(widget);

	painter->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing|QPainter::HighQualityAntialiasing|QPainter::SmoothPixmapTransform, false);

	// Use the renderer to draw the item
	if (_renderer)
		_renderer->render(painter, ItemRenderer::RenderModeFlag::RenderToScene, getRenderFlags());
}

QPainterPath ImageEditorNode::shape() const
{
	// Use the renderer to get the item's shape
	if (_renderer)
	{
		QPainterPath path = _renderer->shape(getRenderFlags());

		if (!path.isEmpty())
			return path;
	}

	return VisualNode::shape();
}

ItemRenderer::RenderFlags ImageEditorNode::getRenderFlags() const
{
	ItemRenderer::RenderFlags flags = ItemRenderer::RenderFlag::NoFlag;

	if (isSelected())
		flags |= ItemRenderer::RenderFlag::Selected;

	if (!_scene->hasFocus())
		flags |= ItemRenderer::RenderFlag::Inactive;

	if (_imageEditor->environment().showDirectionArrows())
		flags |= ItemRenderer::RenderFlag::ShowDirections;

	if (_imageEditor->environment().showTags())
		flags |= ItemRenderer::RenderFlag::ShowTags;

	return flags;
}
