/******************************************************************************
 * File: ImageEditorScene.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORSCENE_H
#define IMAGEEDITORSCENE_H

#include "ui/visscene/VisualScene.h"
#include "ImageEditorView.h"
#include "ImageEditorComponent.h"
#include "DraftItemNodeFactory.h"
#include "ImageEditorTool.h"
#include "image/ImageBuild.h"

namespace grndr
{
	class ImageEditor;
	class ImageEditorNode;
	class LayerPixelsNode;

	class ImageEditorScene : public VisualScene<ImageEditorView>, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		ImageEditorScene(ImageEditor* imageEditor, ImageBuild* imageBuild, ImageEditorView* view);

	public:
		void buildScene();

		void refreshImage(bool centerOnImage = false);
		void fitViewToImage();

		void createDraftItemNode(const std::shared_ptr<DraftItem>& item);
		void removeDraftItemNode(const std::shared_ptr<DraftItem>& item);

		void updateDraftItemsOrder();
		void updateDraftItemsVisibility(const Layer* layer = nullptr);

		void createLayerPixelsNode(const std::shared_ptr<Layer>& layer);
		void removeLayerPixelsNode(const std::shared_ptr<Layer>& layer);

		void updateLayerPixelsOrder();
		void updateLayerPixelsVisibility(const Layer* layer = nullptr);

		void removeNode(QGraphicsItem* node) { if (node) delete node; }

	public:
		DraftItemNode* findDraftItemNode(const DraftItem* item) { return _findDraftItemNode(item); }
		const DraftItemNode* findDraftItemNode(const DraftItem* item) const { return _findDraftItemNode(item); }
		std::vector<DraftItemNode*> findDraftItemNodes(const Layer* layer = nullptr) { return _findDraftItemNodes<DraftItemNode*>(layer); }
		std::vector<const DraftItemNode*> findDraftItemNodes(const Layer* layer = nullptr) const { return _findDraftItemNodes<const DraftItemNode*>(layer); }

		LayerPixelsNode* findLayerPixelsNode(const LayerPixels* layerPixels) { return _findLayerPixelsNode(layerPixels); }
		const LayerPixelsNode* findLayerPixelsNode(const LayerPixels* layerPixels) const { return _findLayerPixelsNode(layerPixels); }
		std::vector<LayerPixelsNode*> findLayerPixelsNodes(const Layer* layer = nullptr) { return _findLayerPixelsNodes<LayerPixelsNode*>(layer); }
		std::vector<const LayerPixelsNode*> findLayerPixelsNodes(const Layer* layer = nullptr) const { return _findLayerPixelsNodes<const LayerPixelsNode*>(layer); }

	public:
		ImageBuild* imageBuild() { return _imageBuild; }
		const ImageBuild* imageBuild() const { return _imageBuild; }

		const DraftItemNodeFactory& nodeFactory() const { return _nodeFactory; }

	protected:
		virtual void mousePressEvent(QGraphicsSceneMouseEvent* mouseEvent) override;
		virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* mouseEvent) override;
		virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* mouseEvent) override;

		virtual void keyPressEvent(QKeyEvent* keyEvent) override;
		virtual void keyReleaseEvent(QKeyEvent* keyEvent) override;

	private:
		void createBackgroundNode();

		ImageEditorTool::InputEventResult handleMouseEvent(QGraphicsSceneMouseEvent* mouseEvent, std::function<ImageEditorTool::InputEventResult(ImageEditorTool*, QGraphicsSceneMouseEvent*)> handler);
		ImageEditorTool::InputEventResult handleKeyEvent(QKeyEvent* keyEvent, std::function<ImageEditorTool::InputEventResult(ImageEditorTool*, QKeyEvent*)> handler);

	private:
		DraftItemNode* _findDraftItemNode(const DraftItem* item) const;
		template<typename ValType>
		std::vector<ValType> _findDraftItemNodes(const Layer* layer = nullptr) const;

		LayerPixelsNode* _findLayerPixelsNode(const LayerPixels* layerPixels) const;
		template<typename ValType>
		std::vector<ValType> _findLayerPixelsNodes(const Layer* layer = nullptr) const;

	private:
		ImageBuild* _imageBuild{nullptr};

		DraftItemNodeFactory _nodeFactory;

	private:
		QGraphicsPixmapItem* _imageNode{nullptr};
	};
}

#include "ImageEditorScene.impl.h"

#endif
