/******************************************************************************
 * File: ImageEditorTool.h
 * Date: 22.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORTOOL_H
#define IMAGEEDITORTOOL_H

#include <QIcon>
#include <QCursor>
#include <QKeySequence>

#include "common/properties/PropertyObject.h"
#include "ui/visscene/VisualSceneInputHandler.h"
#include "ImageEditorComponent.h"

namespace grndr
{
	class ImageEditor;

	class ImageEditorTool : public PropertyObject, public ImageEditorComponent, public VisualSceneInputHandler
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Type;

	public:
		ImageEditorTool(ImageEditor* imageEditor, QString name, QString icon, QString shortcut, QCursor cursor = QCursor{Qt::ArrowCursor});

	public:
		void initImageEditorTool();

	public:
		virtual void toolActivated(ImageEditorTool* prevTool);
		virtual void toolDeactivated(ImageEditorTool* nextTool);

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	public:
		virtual QString getToolType() const = 0;

		QString getName() const { return _name; }
		QIcon getIcon() const { return _icon; }
		QString getShortcut() const { return _shortcut; }
		bool isActionShortcut() const { return _isActionShortcut; }
		QCursor getCursor() const { return _cursor; }

	protected:
		QString _name{""};
		QIcon _icon;
		QString _shortcut;
		bool _isActionShortcut{true};
		QCursor _cursor;

		bool _supportRightMouseButton{false};

	private:
		Qt::ContextMenuPolicy _prevContextMenuPolicy;
	};
}

#endif
