/******************************************************************************
 * File: ImageEditorView.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORVIEW_H
#define IMAGEEDITORVIEW_H

#include "ui/visscene/VisualSceneView.h"
#include "ImageEditorStyle.h"

namespace grndr
{
	class ImageEditorScene;
	class ImageEditorTool;

	class ImageEditorView : public VisualSceneView
	{
		Q_OBJECT

	public:
		ImageEditorView(QWidget *parent = nullptr);

	public:
		ImageEditorScene* editorScene() { return _editorScene; }
		const ImageEditorScene* editorScene() const { return _editorScene; }
		void setEditorScene(ImageEditorScene* scene);

	public slots:
		virtual void removeSelectedItems() const override;

	public:
		virtual const ImageEditorStyle& sceneStyle() const override { return _editorStyle; }

		virtual void prepareNodeContextMenu(QMenu& menu) const override;
		virtual void prepareNodesContextMenu(QMenu& menu) const override;

	protected:
		virtual std::vector<QAction*> getActions(AddActionsMode mode) const override;

	protected:
		virtual void drawBackground(QPainter* painter, const QRectF& rect) override;

	protected slots:
		virtual void updateActions() override;

	private slots:
		void showDirectionArrows();
		void showTags();
		void fitImageToWindow();

		void imageEditorToolChanged(const ImageEditorTool* tool);
		void showDirectionArrowsChanged(bool show);
		void showTagsChanged(bool show);

		void copyDraftItems() const;
		void pasteDraftItems() const;
		void cutDraftItems() const;
		void convertDraftItems() const;

	private:
		ImageEditorScene* _editorScene{nullptr};

		ImageEditorStyle _editorStyle;

	private:
		QAction* _showDirectionsAction{nullptr};
		QAction* _showTagsAction{nullptr};
		QAction* _zoomFitAction{nullptr};

		QAction* _copyDraftItems{nullptr};
		QAction* _pasteDraftItems{nullptr};
		QAction* _cutDraftItems{nullptr};
		QAction* _convertDraftItemsAction{nullptr};
	};
}

#endif
