/******************************************************************************
 * File: ImageTagsListWidget.cpp
 * Date: 01.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageTagsListWidget.h"
#include "pipeline/Block.h"
#include "image/properties/ImageTagsProperty.h"
#include "controller/ImageEditorController.h"
#include "ui/image/ImageEditor.h"
#include "res/Resources.h"

ImageTagsListWidget::ImageTagsListWidget(QWidget* parent) : BaseImageTagsListWidget(parent),
	_dragInfo{this, IMAGETAGSLIST_DRAG_MIMETYPE, FILE_ICON_TAGS}
{
	_dragInfo.setDropAction(Qt::LinkAction);
}

void ImageTagsListWidget::assignUiComponents(ImageEditor* imageEditor)
{
	if (!imageEditor)
		throw std::invalid_argument{_EXCPT("imageEditor may not be null")};

	_imageEditor = imageEditor;

	// Listen for controller events in order to update the UI accordingly
	connect(&imageEditor->controller(), &ImageEditorController::imageBuildSwitched, this, &ImageTagsListWidget::imageBuildSwitched);
}

ImageTagsListItem* ImageTagsListWidget::addObject(ImageTag* imageTag, bool autoSelect, bool insertAtFront)
{
	// Enable dragging of items in this list
	auto item = BaseImageTagsListWidget::addObject(imageTag, autoSelect, insertAtFront);
	item->setFlags(item->flags()|Qt::ItemIsDragEnabled);
	return item;
}

void ImageTagsListWidget::startDrag(Qt::DropActions supportedActions)
{
	Q_UNUSED(supportedActions);

	// Get all selected image tags
	QStringList imageTags;

	for (const auto& selItem : selectedItems())
	{
		if (auto item = dynamic_cast<const ImageTagsListItem*>(selItem))
			imageTags << item->getName();
	}

	// Start dragging
	_dragInfo.dragData().setValue(imageTags.join("\n"));
	_dragInfo.drag();
}

void ImageTagsListWidget::imageBuildSwitched(ImageBuild* imageBuild)
{
	// Remove any previously connected signals from the image build's block
	if (_imageBuild)
		disconnect(_imageBuild, nullptr, this, nullptr);

	_imageBuild = imageBuild;

	// Listen for connection signals from the block's image tags in-port
	if (_imageBuild)
		connect(_imageBuild, SIGNAL(inputImageTagsChanged(const ImageTags*)), this, SLOT(populateList()));

	// A new image build is shown in the editor, so populate its associated image tags
	populateList();
}

void ImageTagsListWidget::populateList()
{
	assignImageTags(_imageBuild ? _imageBuild->inputImageTags() : nullptr);
}
