/******************************************************************************
 * File: InPlaceEditorDragHandle.h
 * Date: 28.3.2018
 *****************************************************************************/

#ifndef INPLACEEDITORDRAGHANDLE_H
#define INPLACEEDITORDRAGHANDLE_H

#include "ImageEditorNode.h"
#include "ImageEditorStyle.h"

namespace grndr
{
	class InPlaceEditorDragHandle : public ImageEditorNode
	{
		Q_OBJECT

	public:
		enum AllowedDirection
		{
			NorthSouth = 0x01,
			EastWest = 0x02,

			All = NorthSouth|EastWest,
		};

		Q_DECLARE_FLAGS(AllowedDirections, AllowedDirection)

	public:
		InPlaceEditorDragHandle(ImageEditorScene* scene, AllowedDirections allowedDirections, QGraphicsItem* parent = nullptr);

	public:
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

	signals:
		void handlePressed();
		void handleMoved(QPoint);
		void handleReleased();

	protected:
		virtual void mousePressEvent(QGraphicsSceneMouseEvent* event) override;
		virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* event) override;
		virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event) override;

	protected:
		virtual void updateGeometry() override;

	private:
		QPoint restrictMovement(QPointF offset) const;

	private:
		const ImageEditorStyle::DragHandleStyle& _dragHandleStyle;

		AllowedDirections _allowedDirections{AllowedDirection::All};

		struct
		{
			bool isDragging{false};
			QPointF lastMovementPosition;
		} _dragInfo;
	};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::InPlaceEditorDragHandle::AllowedDirections)

#endif
