/******************************************************************************
 * File: LayerPixelsNode.cpp
 * Date: 18.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LayerPixelsNode.h"
#include "ImageEditorStyle.h"
#include "image/Layer.h"

LayerPixelsNode::LayerPixelsNode(ImageEditorScene* scene, const std::shared_ptr<Layer>& layer, QGraphicsItem* parent) : ImageEditorNode(scene, parent),
	_layer{layer}
{
	setFlags(0);	// This item doesn't behave like an item at all

	if (auto layer = _layer.lock())	// Make sure that the underlying layer still exists
	{
		// Create the renderer for the layer pixels
		_renderer = layer->layerPixels().createRenderer(_style.getItemNodeStyle());

		// Update the node whenever the pixels data has changed
		connect(&layer->layerPixels().data(), &LayerPixelsData::dataModified, this, &LayerPixelsNode::updateNode);
	}

	updateZOrder();
	updateVisibility();

	updateNode();
}

void LayerPixelsNode::updateNode()
{
	prepareGeometryChange();
	updateGeometry();

	update();
}

void LayerPixelsNode::updateZOrder()
{
	if (auto layer = _layer.lock())	// Make sure that the underlying layer still exists
		setZValue(layer->getZOrder() - 0.1);	// Place the pixels just below all other items
}

void LayerPixelsNode::updateVisibility()
{
	if (auto layer = _layer.lock())	// Make sure that the underlying layer still exists
		setVisible(layer->isVisible());	// Use the visibility flag of the layer
}

void LayerPixelsNode::updateGeometry()
{
	if (auto layer = _layer.lock())	// Make sure that the underlying layer still exists
		_nodeRect = _nodeRectSelected = layer->layerPixels().data().bounds();

	ImageEditorNode::updateGeometry();
}
