/******************************************************************************
 * File: RectangularInPlaceEditor.cpp
 * Date: 29.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "RectangularInPlaceEditor.h"

#define DRAG_HANDLE_COUNT 8

RectangularInPlaceEditor::RectangularInPlaceEditor(ImageEditorScene* scene, DraftItemNode* draftItemNode, PointProperty* position, SizeProperty* size) : InPlaceEditor(scene, draftItemNode),
	_position{position}, _size{size}
{
	if (!position)
		throw std::invalid_argument{_EXCPT("position may not be null")};

	if (!size)
		throw std::invalid_argument{_EXCPT("size may not be null")};

	// Create all 8 drag handles, starting with the top-left one and advancing clockwise
	for (int i = 1; i <= DRAG_HANDLE_COUNT; ++i)
	{
		InPlaceEditorDragHandle::AllowedDirection direction = InPlaceEditorDragHandle::AllowedDirection::All;

		if (i % 4 == 0)
			direction = InPlaceEditorDragHandle::AllowedDirection::EastWest;
		else if (i % 2 == 0)
			direction = InPlaceEditorDragHandle::AllowedDirection::NorthSouth;

		_dragHandles.push_back(createDragHandle(direction));
	}
}

void RectangularInPlaceEditor::updateEditor()
{
	updateDragHandlePositions();
}

void RectangularInPlaceEditor::dragHandleMoved(InPlaceEditorDragHandle* dragHandle, QPoint offset)
{
	QSize sizeOffset{0, 0};
	QPoint posOffset{0, 0};

	auto it = std::find(_dragHandles.cbegin(), _dragHandles.cend(), dragHandle);

	if (it != _dragHandles.cend())
	{
		auto handleIndex = it - _dragHandles.cbegin();

		// Set the offset value depending on where the drag handle is located
		switch (handleIndex)
		{
		case 0: case 1: case 7:
			posOffset = offset;
			sizeOffset = QSize{-offset.x(), -offset.y()};
			break;

		case 2:
			posOffset = QPoint{0, offset.y()};
			sizeOffset = QSize{offset.x(), -offset.y()};
			break;

		case 3: case 4: case 5:
			sizeOffset = QSize{offset.x(), offset.y()};
			break;

		case 6:
			posOffset = QPoint{offset.x(), 0};
			sizeOffset = QSize{-offset.x(), offset.y()};
			break;
		}
	}

	_position->setValue(*_position + posOffset);
	_size->setValue(*_size + sizeOffset);
}

void RectangularInPlaceEditor::updateDragHandlePositions()
{
	// Create a vector containing all relative drag handle positions
	static const std::vector<QPointF> relativePositions{{0.0, 0.0}, {0.5, 0.0}, {1.0, 0.0}, {1.0, 0.5}, {1.0, 1.0}, {0.5, 1.0}, {0.0, 1.0}, {0.0, 0.5}};

	// Move all drag handles to their relative positions
	QSize size = *_size;

	for (unsigned int i = 0; i < _dragHandles.size(); ++i)
	{
		QPointF pos{size.width() * relativePositions[i].x(), size.height() * relativePositions[i].y()};
		_dragHandles[i]->setPos(pos);
	}
}

