/******************************************************************************
 * File: ColorPickerTool.cpp
 * Date: 22.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ColorPickerTool.h"
#include "ui/image/ImageEditor.h"
#include "res/Resources.h"

const char* ColorPickerTool::tool_type = "ColorPickerTool";

ColorPickerTool::ColorPickerTool(ImageEditor* imageEditor) : ImageEditorTool(imageEditor, "Color picker", FILE_ICON_EDITOR_COLORPICKER, "K", QCursor{QPixmap{FILE_CURSOR_EDITOR_COLORPICKER}, 0, 23})
{

}

void ColorPickerTool::toolActivated(ImageEditorTool* prevTool)
{
	ImageEditorTool::toolActivated(prevTool);

	_previousTool = prevTool;
	_switchToPreviousTool = false;
}

ImageEditorTool::InputEventResult ColorPickerTool::mousePressed(const QGraphicsSceneMouseEvent* event)
{
	if (event->widget())
	{
		// Grab the current scene display and get the pixel color under the cursor		
		auto pixmap = event->widget()->grab();
		auto image = pixmap.toImage();
		auto pos = event->widget()->mapFromGlobal(event->screenPos());

		// Set the grabbed color as the primary one and switch back to the previous tool
		_imageEditor->environment().setPrimaryColor(image.pixelColor(pos));
		_switchToPreviousTool = true;
	}

	return InputEventResult::Process;
}

ImageEditorTool::InputEventResult ColorPickerTool::mouseMoved(const QGraphicsSceneMouseEvent* event)
{
	Q_UNUSED(event);
	return InputEventResult::Process;
}

ImageEditorTool::InputEventResult ColorPickerTool::mouseReleased(const QGraphicsSceneMouseEvent* event)
{
	Q_UNUSED(event);

	if (_switchToPreviousTool)
	{
		if (_previousTool)
			_imageEditor->editorTools().activateTool(_previousTool->getToolType());

		_switchToPreviousTool = false;
	}

	return InputEventResult::Process;
}
