/******************************************************************************
 * File: DraftItemTool.h
 * Date: 22.3.2018
 *****************************************************************************/

#ifndef DRAFTITEMTOOL_H
#define DRAFTITEMTOOL_H

#include "ui/image/ImageEditorTool.h"
#include "image/DraftItemType.h"
#include "image/properties/ImageTagsAllotmentProperty.h"

namespace grndr
{
	class DraftItem;

	class DraftItemTool : public ImageEditorTool
	{
		Q_OBJECT

	public:
		DraftItemTool(ImageEditor* imageEditor, DraftItemType itemType, QString name, QString icon, QString shortcut, QCursor cursor = QCursor{Qt::ArrowCursor});

	public:
		DraftItemType getItemType() const { return _itemType; }

	public:
		auto primaryColor() { return dynamic_cast<ColorProperty*>(_primaryColor.get()); }
		auto primaryColor() const { return dynamic_cast<const ColorProperty*>(_primaryColor.get()); }
		auto position() { return dynamic_cast<PointProperty*>(_position.get()); }
		auto position() const { return dynamic_cast<const PointProperty*>(_position.get()); }
		auto hasDirection() { return dynamic_cast<BoolProperty*>(_hasDirection.get()); }
		auto hasDirection() const { return dynamic_cast<const BoolProperty*>(_hasDirection.get()); }
		auto direction() { return dynamic_cast<AngleProperty*>(_direction.get()); }
		auto direction() const { return dynamic_cast<const AngleProperty*>(_direction.get()); }
		auto imageTagsAllotment() { return dynamic_cast<ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }
		auto imageTagsAllotment() const { return dynamic_cast<const ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }

	protected:
		std::shared_ptr<DraftItem> createDraftItem(QPointF pos, bool selectItem, bool setDefaultProperties);
		void selectDraftItem(const std::shared_ptr<DraftItem>& draftItem) const;

	protected:
		virtual void createProperties() override;

		virtual InputEventResult mousePressed(const QGraphicsSceneMouseEvent* event) override;
		virtual InputEventResult mouseMoved(const QGraphicsSceneMouseEvent* event) override;
		virtual InputEventResult mouseReleased(const QGraphicsSceneMouseEvent* event) override;

		virtual InputEventResult keyPressed(const QKeyEvent* event) override;

	private:
		void startDraftItemDrag();
		void cancelDraftItemDrag();

	protected:
		DraftItemType _itemType{DraftItemType::Undefined};

		std::shared_ptr<PropertyBase> _primaryColor;
		std::shared_ptr<PropertyBase> _position;
		std::shared_ptr<PropertyBase> _hasDirection;
		std::shared_ptr<PropertyBase> _direction;
		std::shared_ptr<PropertyBase> _imageTagsAllotment;

	private slots:
		void imageBuildSwitched(ImageBuild* imageBuild);

	private:
		struct
		{
			bool dragInitiated{false};
			bool dragCancelled{false};
			QPointF mousePressPos;
			std::shared_ptr<DraftItem> draftItem;
		} _dragInfo;		
	};
}

#endif
