/******************************************************************************
 * File: EraserTool.cpp
 * Date: 21.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "EraserTool.h"
#include "ui/image/ImageEditor.h"
#include "util/ImageUtils.h"
#include "res/Resources.h"

const char* EraserTool::tool_type = "EraserTool";

EraserTool::EraserTool(ImageEditor* imageEditor) : PaintbrushTool(imageEditor, "Eraser", FILE_ICON_EDITOR_ERASER, "E", QCursor{QPixmap{FILE_CURSOR_EDITOR_ERASER}, 12, 12})
{
	_eraseByDefault = true;
}

void EraserTool::createProperties()
{
	PaintbrushTool::createProperties();

	// The eraser should have a default width equal to the cursor size
	penWidth()->setValue(_cursor.pixmap().width());
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMousePressed(const QGraphicsSceneMouseEvent* event)
{
	// Create a rubber band for selection a region to erase
	if (!_rubberBandNode)
	{
		if (auto scene = _imageEditor->controller().activeScene())
		{
			_rubberBand = new QRubberBand{QRubberBand::Rectangle};
			_rubberBand->move(event->scenePos().toPoint());
			_rubberBand->resize(0, 0);
			_rubberBand->show();

			_rubberBandNode = scene->addWidget(_rubberBand);
			_rubberBandNode->setZValue(std::numeric_limits<qreal>::max());
		}
	}

	return InputEventResult::Process;
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMouseMoved(const QGraphicsSceneMouseEvent* event)
{
	if (_rubberBand)
	{
		auto delta = event->scenePos() - event->buttonDownScenePos(Qt::RightButton);
		QPointF topLeft{std::min(event->scenePos().x(), event->scenePos().x() - delta.x()), std::min(event->scenePos().y(), event->scenePos().y() - delta.y())};
		QRect rect{topLeft.toPoint(), QSizeF{std::abs(delta.x()), std::abs(delta.y())}.toSize()};
		_rubberBand->setGeometry(rect);
	}

	return InputEventResult::Process;
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMouseReleased(const QGraphicsSceneMouseEvent* event)
{
	Q_UNUSED(event);

	if (_rubberBandNode)
	{
		// Erase all pixels within the rubber band rectangle
		auto eraseRect = _rubberBand->geometry();
		std::vector<QPoint> points;

		for (int r = eraseRect.top(); r <= eraseRect.bottom(); ++r)
		{
			for (int c = eraseRect.left(); c <= eraseRect.right(); ++c)
				points.emplace_back(c, r);
		}

		_imageEditor->controller().paintPixels(nullptr, QColor{}, points);

		// Remove the rubber band
		delete _rubberBandNode;
	}

	_rubberBand = nullptr;
	_rubberBandNode = nullptr;

	return InputEventResult::Process;
}
