/******************************************************************************
 * File: PropertyEditor.impl.h
 * Date: 07.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "PropertyEditor.h"
#include "common/properties/PropertyBase.h"

template<typename Base, typename PropertyType>
PropertyEditor<Base, PropertyType>::PropertyEditor(PropertyType* property, QWidget* parent) : Base(parent),
	_property{property}
{
	if (!property)
		throw std::invalid_argument{_EXCPT("property may not be null")};

	this->setFocusPolicy(Qt::StrongFocus);

	// If the property value changes, reflect the new value
	_property->connect(_property, &PropertyBase::valueChanged, this, &widget_type::applyPropertyValue);
}

template<typename Base, typename PropertyType>
void PropertyEditor<Base, PropertyType>::showEvent(QShowEvent* event)
{
	base_type::showEvent(event);
	applyPropertyValue();
}

template<typename Base, typename PropertyType>
QString PropertyEditor<Base, PropertyType>::getPropertyValue() const
{
	return _property->toString();
}

template<typename Base, typename PropertyType>
void PropertyEditor<Base, PropertyType>::setPropertyValue(const QString& value)
{
	try {
		_property->fromString(value);
	} catch (...) {
		// If the string-value is invalid, revert to the previous value in the editor by simulating a property value change
		applyPropertyValue();
	}
}
