/******************************************************************************
 * File: PropertyTreeItemDelegate.cpp
 * Date: 07.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "PropertyTreeItemDelegate.h"
#include "PropertyTreeWidget.h"
#include "ValuePropertyTreeItem.h"

#include "PropertyEditor.h"

PropertyTreeItemDelegate::PropertyTreeItemDelegate(PropertyTreeWidget* widget, QObject* parent) : QStyledItemDelegate(parent),
	_widget{widget}
{
	if (!widget)
		throw std::invalid_argument{_EXCPT("widget may not be null")};
}

QWidget* PropertyTreeItemDelegate::createEditor(QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
	Q_UNUSED(parent);
	Q_UNUSED(option);

	if (auto item = _widget->valuePropertyItem(index))
	{
		if (auto editor = item->createEditor(parent))
		{
			editor->setProperty(PropertyEditorBase::property_font, _widget->font());	// Forward the tree widget's font to the editor as a dynamic property
			return editor;
		}
	}

	return nullptr;
}

void PropertyTreeItemDelegate::setModelData(QWidget* editor, QAbstractItemModel* model, const QModelIndex& index) const
{
	Q_UNUSED(editor);
	Q_UNUSED(model);

	// The model data has been changed, so update the corresponding item
	if (auto item = _widget->valuePropertyItem(index))
		item->updateItem();
}

void PropertyTreeItemDelegate::initStyleOption(QStyleOptionViewItem* option, const QModelIndex& index) const
{
	QStyledItemDelegate::initStyleOption(option, index);

	// If editing the given item, hide its text so that it doesn't appear below the editor
	if (_widget->isEditing() && index.row() == _widget->currentIndex().row())
		option->text = "";
}
