/******************************************************************************
 * File: DialogPropertyEditor.impl.h
 * Date: 18.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DialogPropertyEditor.h"

template<typename PropertyType>
DialogPropertyEditor<PropertyType>::DialogPropertyEditor(PropertyType* property, QWidget* parent) : PropertyEditor<QWidget, PropertyType>(property, parent),
	_valueLabel{new QLabel{}}, _editButton{new QToolButton{}}
{
	this->setFocusPolicy(Qt::NoFocus);

	_valueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
	_valueLabel->setStyleSheet("QTreeWidget QLabel { margin-left: 1px; }");

	_editButton->setText("...");
	_editButton->setToolTip("Edit items");

	// Create a horizontal layout and add all controls to it
	auto layout = new QHBoxLayout{};
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(_valueLabel);
	layout->addWidget(_editButton);
	this->setLayout(layout);

	// Listen for edit button clicks
	this->connect(_editButton, &QToolButton::clicked, this, &DialogPropertyEditor<PropertyType>::editPropertyClicked);

	applyPropertyValue();
}

template<typename PropertyType>
void DialogPropertyEditor<PropertyType>::showEvent(QShowEvent* event)
{
	PropertyEditor<QWidget, PropertyType>::showEvent(event);

	// Set the value label's font to the parent's one
	QVariant fontProperty = this->property(PropertyEditorBase::property_font);
	QFont fnt = fontProperty.value<QFont>();
	_valueLabel->setFont(QFont{fnt.family(), fnt.pointSize()});	// Passing fnt won't work (for whatever reason)
}

template<typename PropertyType>
void DialogPropertyEditor<PropertyType>::mouseDoubleClickEvent(QMouseEvent* event)
{
	PropertyEditor<QWidget, PropertyType>::mouseDoubleClickEvent(event);

	// Invoke the dialog on left double-click
	if (event->button() == Qt::LeftButton)
		invokeDialog();
}

template<typename PropertyType>
void DialogPropertyEditor<PropertyType>::setValueLabelText(QString text)
{
	QFontMetrics fontMetrics(_valueLabel->font());
	_valueLabel->setText(fontMetrics.elidedText(text, Qt::ElideRight, _valueLabel->width() - 2));
	_valueLabel->setToolTip(this->getPropertyValue());
}

template<typename PropertyType>
void DialogPropertyEditor<PropertyType>::applyPropertyValue()
{
	setValueLabelText(this->getPropertyValue());
}

template<typename PropertyType>
void DialogPropertyEditor<PropertyType>::editPropertyClicked(bool checked)
{
	Q_UNUSED(checked);
	invokeDialog();
}
