/******************************************************************************
 * File: ImageReferencePropertyEditor.cpp
 * Date: 12.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageReferencePropertyEditor.h"
#include "core/GrinderApplication.h"

Q_DECLARE_METATYPE(ImageReference*)

ImageReferencePropertyEditor::ImageReferencePropertyEditor(ImageReferenceProperty* property, QWidget *parent) : PropertyEditor(property, parent)
{
	// Fill the image references list
	populateImageReferences();

	// Update the property whenever the current index has been changed
	connect(this, SIGNAL(currentIndexChanged(int)), this, SLOT(currentIndexChanged(int)));

	// React to new or deleted image references
	connect(&grinder()->project(), SIGNAL(imageReferenceCreated(const std::shared_ptr<ImageReference>&)), this, SLOT(populateImageReferences()));
	connect(&grinder()->project(), SIGNAL(imageReferenceRemoved(const std::shared_ptr<ImageReference>&)), this, SLOT(populateImageReferences()));
}

void ImageReferencePropertyEditor::applyPropertyValue()
{
	bool itemSelected = false;

	// Search for the corresponding image reference item in the list
	if (auto imageRef = _property->getValue())
	{
		for (int i = 0; i < count(); ++i)
		{
			auto data = itemData(i);

			if (data.value<ImageReference*>() == imageRef)
			{
				setCurrentIndex(i);
				itemSelected = true;
				break;
			}
		}
	}

	// If no item was selected, select the default (active image) one
	if (!itemSelected)
		setCurrentIndex(0);
}

void ImageReferencePropertyEditor::populateImageReferences()
{
	clear();

	// The first item always represents the currently active image
	QVariant data;
	data.setValue(static_cast<ImageReference*>(nullptr));
	addItem("<Active image>", data);

	auto boldFont = font();
	boldFont.setBold(true);
	setItemData(0, boldFont, Qt::FontRole);

	// Add all image references
	insertSeparator(1);

	for (const auto& imageRef : grinder()->project().imageReferences().sorted())
	{
		QVariant data;
		data.setValue(imageRef.get());
		addItem(imageRef->getImageFileName(), data);
	}

	applyPropertyValue();
}

void ImageReferencePropertyEditor::currentIndexChanged(int index)
{
	_property->setValue(itemData(index).value<ImageReference*>());
}
