/******************************************************************************
 * File: ImageTagsAllotmentPropertyEditor.cpp
 * Date: 18.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageTagsAllotmentPropertyEditor.h"
#include "image/ImageTags.h"
#include "ui/dlg/CheckListDialog.h"
#include "ui/image/ImageTagsListItem.h"

ImageTagsAllotmentPropertyEditor::ImageTagsAllotmentPropertyEditor(ImageTagsAllotmentProperty* property, QWidget* parent) : DialogPropertyEditor(property, parent)
{
	// Reflect updated image tags in the allotment
	connect(&property->object(), &ImageTagsAllotment::allotmentModified, this, &ImageTagsAllotmentPropertyEditor::applyPropertyValue);
}

void ImageTagsAllotmentPropertyEditor::invokeDialog()
{
	CheckListDialog dlg{"Assign tags", true};

	if (auto imageTags = _property->object().inputImageTags())
	{
		bool accepted = false;
		std::vector<ImageTag*> allottedTags{_property->object().allottedTags().cbegin(), _property->object().allottedTags().cend()};
		auto selectedTags = dlg.exec<ImageTag, ImageTagsListItem>(imageTags->tags(), allottedTags, &accepted);

		if (accepted)
		{
			// Assign all selected image tags
			_property->object().unallotAllTags();

			for (auto imageTag : selectedTags)
				_property->object().allotTag(imageTag);

			// The associated image tags have been modified, so notify the property
			_property->objectModified();
		}
	}
	else
		QMessageBox::information(nullptr, "Assigning tags", "There are currently no associated input tags.");
}
