/******************************************************************************
 * File: IntPropertyEditor.cpp
 * Date: 28.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "IntPropertyEditor.h"
#include "common/Range.h"
#include "common/properties/types/RangeConstraint.h"

IntPropertyEditor::IntPropertyEditor(PropertyBase* property, const PropertyConstraints* constraints, QWidget* parent) : PropertyEditor(property, parent)
{
	if (constraints)
		setRangeFromConstraints(*constraints);

	applyPropertyValue();

	// Update the property whenever the value has changed
	connect(this, SIGNAL(valueChanged(int)), this, SLOT(valueChanged()));
}

void IntPropertyEditor::setRangeFromConstraints(const PropertyConstraints& constraints)
{
	for (auto& constraint : constraints)
	{
		// Handle both int and unsigned int
		if (auto rangeConstraint = dynamic_cast<RangeConstraint<int>*>(constraint.get()))
		{
			auto range = rangeConstraint->range();
			setRange(range.first(), range.second());
			break;
		}
		else if (auto rangeConstraint = dynamic_cast<RangeConstraint<unsigned int>*>(constraint.get()))
		{
			auto range = rangeConstraint->range();
			setRange(static_cast<int>(range.first()), static_cast<int>(range.second()));
			break;
		}
	}
}

void IntPropertyEditor::applyPropertyValue()
{
	setValue(getPropertyValue().toInt());
}

void IntPropertyEditor::valueChanged()
{
	setPropertyValue(cleanText());
}
