/******************************************************************************
 * File: VisualNodeFactory.impl.h
 * Date: 21.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "VisualNodeFactory.h"

template<typename NodeType, typename ObjType, typename KeyType, typename SceneType>
VisualNodeFactory<NodeType, ObjType, KeyType, SceneType>::VisualNodeFactory(scene_type* scene) :
	_scene{scene}
{
	if (!scene)
		throw std::invalid_argument{_EXCPT("scene may not be null")};
}

template<typename NodeType, typename ObjType, typename KeyType, typename SceneType>
void VisualNodeFactory<NodeType, ObjType, KeyType, SceneType>::registerType(key_type type, node_creator_type creator)
{
	if (type == key_type::Undefined)
		throw std::invalid_argument{_EXCPT("type may not be Undefined")};

	if (!creator)
		throw std::invalid_argument{_EXCPT("creator may not be null")};

	_nodeCreators[type] = creator;
}

template<typename NodeType, typename ObjType, typename KeyType, typename SceneType>
void VisualNodeFactory<NodeType, ObjType, KeyType, SceneType>::unregisterType(key_type type)
{
	_nodeCreators.erase(type);
}

template<typename NodeType, typename ObjType, typename KeyType, typename SceneType>
NodeType* VisualNodeFactory<NodeType, ObjType, KeyType, SceneType>::createNode(const std::shared_ptr<object_type>& object) const
{
	if (!object)
		throw std::invalid_argument{_EXCPT("object may not be null")};

	key_type type = object->getType();

	if (type == key_type::Undefined)
		throw std::invalid_argument{_EXCPT("type may not be Undefined")};

	// Create a node for the given object (type); if no creator exists, create a basic GraphBlockNode
	if (_nodeCreators.find(type) != _nodeCreators.end())
	{
		node_creator_type creator = _nodeCreators.at(type);
		node_type* node = creator(object);

		if (!node)
			throw std::runtime_error{_EXCPT(QString{"Failed to create an object node of type %1"}.arg(type))};

		return node;
	}
	else	// Use the default block node type
		return createDefaultNode(object);
}
