/******************************************************************************
 * File: VisualScene.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef VISUALSCENE_H
#define VISUALSCENE_H

#include <QGraphicsScene>

namespace grndr
{
	class VisualSceneView;

	template<typename ViewType>
	class VisualScene : public QGraphicsScene
	{
		static_assert(std::is_base_of<VisualSceneView, ViewType>::value, "ViewType must be derived from VisualSceneView");

	public:
		using view_type = ViewType;	

	public:
		VisualScene(view_type* view, QRectF sceneRect = QRectF{});
		virtual ~VisualScene();

	public:
		template<typename NodeType>
		std::vector<NodeType*> getNodes(bool selectedOnly = false) { return _getNodes<NodeType>(selectedOnly); }
		template<typename NodeType>
		std::vector<const NodeType*> getNodes(bool selectedOnly = false) const;

	public:
		view_type* view() { return _view; }
		const view_type* view() const { return _view; }

	protected:
		virtual void drawBackground(QPainter* painter, const QRectF& rect) override;

	private:
		void drawGrid(QPainter* painter, const QRectF& rect, int interval, QColor color);

	private:
		template<typename NodeType>
		std::vector<NodeType*> _getNodes(bool selectedOnly = false) const;

	protected:
		view_type* _view{nullptr};

		bool _drawGrid{true};
	};
}

#include "VisualScene.impl.h"

#endif
