/******************************************************************************
 * File: VisualSceneInputHandler.cpp
 * Date: 28.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "VisualSceneInputHandler.h"

VisualSceneInputHandler::InputEventResult VisualSceneInputHandler::handleMousePressEvent(QGraphicsSceneMouseEvent* event)
{
	_leftMouseButtonPressed = false;
	_rightMouseButtonPressed = false;

	if (!event->modifiers().testFlag(Qt::ShiftModifier))	// Shift-clicks should always be passed to the parent (to support shift-dragging of the scene)
	{
		if (event->button() == Qt::LeftButton)
		{
			_leftMouseButtonPressed = true;
			return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::mousePressed);
		}
		else if (event->button() == Qt::RightButton)
		{
			_rightMouseButtonPressed = true;
			return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::rightMousePressed);
		}
	}

	return InputEventResult::Ignore;
}

VisualSceneInputHandler::InputEventResult VisualSceneInputHandler::handleMouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
	if (_leftMouseButtonPressed)
		return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::mouseMoved);
	else if (_rightMouseButtonPressed)
		return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::rightMouseMoved);
	else
		return InputEventResult::Ignore;
}

VisualSceneInputHandler::InputEventResult VisualSceneInputHandler::handleMouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
	if (event->button() == Qt::LeftButton)
	{
		_leftMouseButtonPressed = false;
		return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::mouseReleased);
	}
	else if (event->button() == Qt::RightButton)
	{
		_rightMouseButtonPressed = false;
		return handleInputEvent<QGraphicsSceneMouseEvent>(event, &VisualSceneInputHandler::rightMouseReleased);
	}
	else
		return InputEventResult::Ignore;
}

VisualSceneInputHandler::InputEventResult VisualSceneInputHandler::handleKeyPressEvent(QKeyEvent* event)
{
	return handleInputEvent<QKeyEvent>(event, &VisualSceneInputHandler::keyPressed);
}

VisualSceneInputHandler::InputEventResult VisualSceneInputHandler::handleKeyReleaseEvent(QKeyEvent* event)
{
	return handleInputEvent<QKeyEvent>(event, &VisualSceneInputHandler::keyReleased);
}
