/******************************************************************************
 * File: VisualSceneInputHandler.h
 * Date: 28.3.2018
 *****************************************************************************/

#ifndef VISUALSCENEINPUTHANDLER_H
#define VISUALSCENEINPUTHANDLER_H

#include <QGraphicsSceneMouseEvent>
#include <QKeyEvent>
#include <functional>

namespace grndr
{
	class VisualSceneInputHandler
	{
	public:
		enum class InputEventResult
		{
			Ignore,
			Process,
		};

	public:
		InputEventResult handleMousePressEvent(QGraphicsSceneMouseEvent* event);
		InputEventResult handleMouseMoveEvent(QGraphicsSceneMouseEvent* event);
		InputEventResult handleMouseReleaseEvent(QGraphicsSceneMouseEvent* event);

		InputEventResult handleKeyPressEvent(QKeyEvent* event);
		InputEventResult handleKeyReleaseEvent(QKeyEvent* event);

	protected:
		virtual InputEventResult mousePressed(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult mouseMoved(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult mouseReleased(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult rightMousePressed(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult rightMouseMoved(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult rightMouseReleased(const QGraphicsSceneMouseEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }

		virtual InputEventResult keyPressed(const QKeyEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }
		virtual InputEventResult keyReleased(const QKeyEvent* event) { Q_UNUSED(event); return InputEventResult::Ignore; }

	private:
		template<typename EventType>
		InputEventResult handleInputEvent(EventType* event, std::function<InputEventResult(VisualSceneInputHandler*, const EventType*)> handler);

	private:
		bool _leftMouseButtonPressed{false};
		bool _rightMouseButtonPressed{false};
	};
}

#include "VisualSceneInputHandler.impl.h"

#endif
