/******************************************************************************
 * File: VisualSceneView.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef VISUALSCENEVIEW_H
#define VISUALSCENEVIEW_H

#include <QGraphicsView>

#include "ui/widgets/MetaWidget.h"
#include "VisualSceneStyle.h"

namespace grndr
{
	class VisualSceneView : public MetaWidget<QGraphicsView>
	{
		Q_OBJECT

	public:
		VisualSceneView(QWidget* parent = nullptr);

	public:
		void setScene(QGraphicsScene* scene);

		qreal getZoom() const;
		void setZoom(qreal zoomVal);
		void zoom(qreal factor);		

	public slots:
		virtual void selectAllItems() const;
		virtual void moveSelectedItems(QPoint delta);
		virtual void removeSelectedItems() const { }

		void zoomIn() { zoom(sceneStyle().getViewStyle().zoomFactor); }
		void zoomOut() { zoom(1.0 / sceneStyle().getViewStyle().zoomFactor); }
		void zoomFull() { setZoom(1.0); }
		void fitToWindow(QGraphicsItem* item = nullptr);		

	public:
		virtual const VisualSceneStyle& sceneStyle() const = 0;

		virtual void prepareNodeContextMenu(QMenu& menu) const { Q_UNUSED(menu); }
		virtual void prepareNodesContextMenu(QMenu& menu) const { Q_UNUSED(menu); }

	signals:
		void zoomChanged(qreal);

	protected:
		virtual void contextMenuEvent(QContextMenuEvent* event) override;
		virtual void wheelEvent(QWheelEvent* event) override;
		virtual void keyPressEvent(QKeyEvent* event) override;
		virtual void keyReleaseEvent(QKeyEvent* event) override;
		virtual void mouseMoveEvent(QMouseEvent* event) override;

	protected slots:
		virtual void updateActions();

	protected:
		QGraphicsScene* _scene{nullptr};

	protected:
		QAction* _selectAllAction{nullptr};
		QAction* _deleteSelectedAction{nullptr};

		QAction* _zoomInAction{nullptr};
		QAction* _zoomOutAction{nullptr};
		QAction* _zoomFullAction{nullptr};		
	};
}

#endif
