/******************************************************************************
 * File: ColorWidget.cpp
 * Date: 24.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ColorWidget.h"

ColorWidget::ColorWidget(QWidget* parent) : QWidget(parent),
	_color{255, 255, 255}
{
	setFocusPolicy(Qt::NoFocus);
	updateWidget();
}

ColorWidget::ColorWidget(Flags flags, QWidget* parent) : ColorWidget(parent)
{
	_flags = flags;
}

QSize ColorWidget::sizeHint() const
{
	// Hardcoded size hints
	if (_flags.testFlag(Flag::SmallBox))
		return QSize{22, 15};
	else
		return QSize{50, 35};
}

void ColorWidget::paintEvent(QPaintEvent* event)
{
	Q_UNUSED(event);

	QRect rc = rect() + QMargins{0, 0, -1, -1};
	QPainter painter{this};
	painter.setRenderHints(QPainter::Antialiasing, false);

	// Draw the background
	QPen pen{Qt::black, 1};
	painter.setPen(pen);

	painter.setBrush(Qt::white);
	painter.drawRect(rc);

	// Draw the current color(s)
	rc -= QMargins{2, 2, 1, 1};
	painter.fillRect(rc, _color);

	if (_showColorComp)
	{
		rc.setLeft(rc.left() + std::ceil(rc.width() / 2.0));
		painter.fillRect(rc, _colorComp);
	}
}

void ColorWidget::mouseReleaseEvent(QMouseEvent* event)
{
	if (event->button() == Qt::LeftButton && event->modifiers() == 0)
	{
		emit colorClicked(_color);

		if (!_flags.testFlag(Flag::SelectColorOnDoubleClick))
		{
			selectColor();
			return;
		}
	}

	QWidget::mouseReleaseEvent(event);
}

void ColorWidget::mouseDoubleClickEvent(QMouseEvent* event)
{
	if (event->button() == Qt::LeftButton && event->modifiers() == 0)
	{
		if (_flags.testFlag(Flag::SelectColorOnDoubleClick))
		{
			selectColor();
			return;
		}
	}

	QWidget::mouseDoubleClickEvent(event);
}

void ColorWidget::selectColor()
{
	bool prevShowColorComp = _showColorComp;
	QColor prevColorComp = _colorComp;

	setColorComp(true, _color);

	QColorDialog colorDlg{_color};
	colorDlg.setOptions(QColorDialog::DontUseNativeDialog);
	connect(&colorDlg, SIGNAL(currentColorChanged(const QColor&)), this, SLOT(colorDialogColorChanged(const QColor&)));

	if (colorDlg.exec() == QDialog::Accepted)
	{
		setColor(colorDlg.currentColor());
		emit colorChanged(_color, true);
	}

	setColorComp(prevShowColorComp, prevColorComp);
}

void ColorWidget::updateWidget()
{
	QString toolTip = QString{"R: %1; G: %2; B: %3<br>%4"}.arg(_color.red()).arg(_color.green()).arg(_color.blue()).arg(_color.name());

	if (!_extraToolTip.isEmpty())
		toolTip.insert(0, QString{"%1<br>"}.arg(_extraToolTip));

	setToolTip(toolTip);
	update();
}

void ColorWidget::colorDialogColorChanged(const QColor& color)
{
	setColorComp(true, color);
}
