/******************************************************************************
 * File: CVUtils.cpp
 * Date: 19.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "CVUtils.h"
#include "image/ImageExceptions.h"

QImage CVUtils::matrixToImage(const cv::Mat& matrix)
{
	cv::Mat imageMatrix = matrix;

	// Check if the matrix has an unsupported type; if so, convert it first
	if (matrix.type() != CV_8UC4 && matrix.type() != CV_8UC3 && matrix.type() != CV_8UC1)
	{
		matrix.convertTo(imageMatrix, CV_8U);
		cv::normalize(imageMatrix, imageMatrix, std::numeric_limits<unsigned char>::max(), std::numeric_limits<unsigned char>::min(), cv::NORM_MINMAX);
	}

	switch (imageMatrix.type())
	{
	case CV_8UC4:
		return QImage(imageMatrix.data, imageMatrix.cols, imageMatrix.rows, static_cast<int>(imageMatrix.step), QImage::Format_ARGB32);

	case CV_8UC3:
		return QImage(imageMatrix.data, imageMatrix.cols, imageMatrix.rows, static_cast<int>(imageMatrix.step), QImage::Format_RGB888).rgbSwapped();

	case CV_8UC1:
		return QImage(imageMatrix.data, imageMatrix.cols, imageMatrix.rows, static_cast<int>(imageMatrix.step), QImage::Format_Grayscale8);

	default:
		throw ImageException{_EXCPT("Unsupported matrix format for conversion to an image")};
	}
}

QPixmap CVUtils::matrixToPixmap(const cv::Mat& matrix)
{
	return QPixmap::fromImage(matrixToImage(matrix));
}
