/******************************************************************************
 * File: StringConv.cpp
 * Date: 12.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "StringConv.h"

/* convertValue */

template<>
QString StringConv::convertValue<bool>(const bool& val)
{
	return val ? "true" : "false";
}

template<>
QString StringConv::convertValue<QPoint>(const QPoint& val)
{
	return QString{"%1;%2"}.arg(val.x()).arg(val.y());
}

template<>
QString StringConv::convertValue<QPointF>(const QPointF& val)
{
	return QString{"%1;%2"}.arg(val.x()).arg(val.y());
}

template<>
QString StringConv::convertValue<QSize>(const QSize& val)
{
	QPoint pt{val.width(), val.height()};
	return convertValue(pt);
}

template<>
QString StringConv::convertValue<QColor>(const QColor& val)
{
	return val.name();
}

/* convertString */

template<>
QString StringConv::convertString<QString>(const QString& str, bool* ok)
{
	if (ok)
		*ok = true;

	return str;
}

template<>
short StringConv::convertString<short>(const QString& str, bool* ok)
{
	return str.toShort(ok);
}

template<>
ushort StringConv::convertString<ushort>(const QString& str, bool* ok)
{
	return str.toUShort(ok);
}

template<>
int StringConv::convertString<int>(const QString& str, bool* ok)
{
	return str.toInt(ok);
}

template<>
uint StringConv::convertString<uint>(const QString& str, bool* ok)
{
	return str.toUInt(ok);
}

template<>
long StringConv::convertString<long>(const QString& str, bool* ok)
{
	return str.toLong(ok);
}

template<>
ulong StringConv::convertString<ulong>(const QString& str, bool* ok)
{
	return str.toULong(ok);
}

template<>
qlonglong StringConv::convertString<qlonglong>(const QString& str, bool* ok)
{
	return str.toLongLong(ok);
}

template<>
qulonglong StringConv::convertString<qulonglong>(const QString& str, bool* ok)
{
	return str.toULongLong(ok);
}

template<>
float StringConv::convertString<float>(const QString& str, bool* ok)
{
	return str.toFloat(ok);
}

template<>
double StringConv::convertString<double>(const QString& str, bool* ok)
{
	return str.toDouble(ok);
}

template<>
bool StringConv::convertString<bool>(const QString& str, bool* ok)
{
	if (ok)
		*ok = true;

	return (str.compare("true", Qt::CaseInsensitive) == 0 || str.compare("yes", Qt::CaseInsensitive) == 0 || str == "1") ? true : false;
}

template<>
QPoint StringConv::convertString<QPoint>(const QString& str, bool* ok)
{
	auto tokens = str.split(";");

	if (tokens.size() == 2)
	{
		bool ok1 = true, ok2 = true;
		QPoint point{tokens[0].toInt(&ok1), tokens[1].toInt(&ok2)};

		if (ok)
			*ok = ok1 && ok2;

		return point;
	}
	else
		return QPoint{};
}

template<>
QPointF StringConv::convertString<QPointF>(const QString& str, bool* ok)
{
	auto tokens = str.split(";");

	if (tokens.size() == 2)
	{
		bool ok1 = true, ok2 = true;
		QPointF point{tokens[0].toDouble(&ok1), tokens[1].toDouble(&ok2)};

		if (ok)
			*ok = ok1 && ok2;

		return point;
	}
	else
		return QPointF{};
}

template<>
QSize StringConv::convertString<QSize>(const QString& str, bool* ok)
{
	QPoint pt = convertString<QPoint>(str, ok);
	return QSize{pt.x(), pt.y()};
}

template<>
QColor StringConv::convertString<QColor>(const QString& str, bool* ok)
{
	QColor color{str};

	if (ok)
		*ok = color.isValid();

	return color;
}
