/******************************************************************************
 * File: UIUtils.cpp
 * Date: 02.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "UIUtils.h"
#include "core/GrinderApplication.h"

QAction* UIUtils::createAction(QObject* owner, QString name, QString icon, const char* callback, QString help, QString shortcut, Qt::ShortcutContext context, QObject* receiver)
{
	auto action = new QAction{name, owner};

	if (auto widget = dynamic_cast<QWidget*>(owner))
		widget->addAction(action);

	if (!icon.isEmpty())
		action->setIcon(QIcon{icon});

	if (!help.isEmpty())
	{
		action->setStatusTip(help);
		action->setToolTip(help);
	}

	if (!shortcut.isEmpty())
		action->setShortcut(QKeySequence{shortcut});

	action->setShortcutContext(context);
	action->setShortcutVisibleInContextMenu(true);

	if (callback)
		owner->connect(action, SIGNAL(triggered(bool)), receiver ? receiver : owner, callback);

	return action;
}

QString UIUtils::askFileName(bool saveFileName, QString dlgName, QWidget* parent, QString caption, QString filter, QString* selectedFilter, QFileDialog::Options options)
{
	QString dir = grinder()->settings().getFileDialogDir(dlgName);
	QString fileName = saveFileName ? QFileDialog::getSaveFileName(parent, caption, dir, filter, selectedFilter, options) : QFileDialog::getOpenFileName(parent, caption, dir, filter, selectedFilter, options);

	if (!fileName.isEmpty())
	{
		QFileInfo fileInfo{fileName};
		grinder()->settings().setFileDialogDir(dlgName, fileInfo.path());
	}

	return fileName;
}

QStringList UIUtils::askFileNames(QString dlgName, QWidget* parent, QString caption, QString filter, QString* selectedFilter, QFileDialog::Options options)
{
	QString dir = grinder()->settings().getFileDialogDir(dlgName);
	auto fileNames = QFileDialog::getOpenFileNames(parent, caption, dir, filter, selectedFilter, options);

	if (!fileNames.isEmpty())
	{
		QFileInfo fileInfo{fileNames.front()};
		grinder()->settings().setFileDialogDir(dlgName, fileInfo.path());
	}

	return fileNames;
}

void UIUtils::removeChildrenFromLayout(QLayout* layout)
{
	QLayoutItem* item;
	QLayout* sublayout;
	QWidget* widget;

	while ((item = layout->takeAt(0)))
	{
		if ((sublayout = item->layout()))
		{
			removeChildrenFromLayout(sublayout);
		}
		else if ((widget = item->widget()))
		{
			widget->hide();
			delete widget;
		}
		else
			delete item;
	}
}
